#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from string import Template


class Help(object):

    ruler = '='

    @staticmethod
    def format(text, subst):
        """Format a command help text, and make '$' substitutions."""
        lines = text.splitlines()
        if not lines:
            return ''
        indent = Help.__indent_level(lines[0])
        for ix in range(len(lines)):
            line = lines[ix][indent:]
            if line.startswith('==') and line.endswith('=='):
                line = line[2:-2].strip().upper()
            else:
                line = '  ' + line
            if '$' in line:
                template = Template(line)
                line = template.safe_substitute(subst)
                if '\n' in line:
                    lindent = Help.__indent_level(line)
                    line = line.replace('\n', '\n' + ' ' * lindent)
            lines[ix] = line
        text = '\n' + '\n'.join(lines) + '\n'
        return text

    @staticmethod
    def __indent_level(s):
        """INTERNAL: return the indentation level of a string."""
        for i in range(len(s)):
            if not s[i].isspace():
                return i
        return len(s)

    @staticmethod
    def format_topics(header, cmds, cmdlen, maxcol):
        txt = ''
        if cmds:
            txt += "%s\n" % str(header)
            if Help.ruler:
                txt += "%s\n" % str(Help.ruler * len(header))
            Help.__columnize(cmds, txt, maxcol - 1)
            txt += "\n"
        return txt

    @staticmethod
    def __columnize(list, txt, displaywidth=80):
        """Display a list of strings as a compact set of columns.

        Each column is only as wide as necessary.
        Columns are separated by two spaces (one was not legible enough).
        """
        if not list:
            txt += "<empty>\n"
            return
        nonstrings = [i for i in range(len(list))
                        if not isinstance(list[i], str)]
        if nonstrings:
            raise TypeError, ("list[i] not a string for i in %s" %
                              ", ".join(map(str, nonstrings)))
        size = len(list)
        if size == 1:
            txt += '%s\n' % str(list[0])
            return
        # Try every row count from 1 upwards
        for nrows in range(1, len(list)):
            ncols = (size + nrows - 1) // nrows
            colwidths = []
            totwidth = -2
            for col in range(ncols):
                colwidth = 0
                for row in range(nrows):
                    i = row + nrows * col
                    if i >= size:
                        break
                    x = list[i]
                    colwidth = max(colwidth, len(x))
                colwidths.append(colwidth)
                totwidth += colwidth + 2
                if totwidth > displaywidth:
                    break
            if totwidth <= displaywidth:
                break
        else:
            nrows = len(list)
            ncols = 1
            colwidths = [0]
        for row in range(nrows):
            texts = []
            for col in range(ncols):
                i = row + nrows * col
                if i >= size:
                    x = ""
                else:
                    x = list[i]
                texts.append(x)
            while texts and not texts[-1]:
                del texts[-1]
            for col in range(len(texts)):
                texts[col] = texts[col].ljust(colwidths[col])
            txt += "%s\n" % str("  ".join(texts))



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


def create(cls, *args, **kwargs):
    """Create ovirtcli objects."""
    from ovirtcli.format import Formatter, get_formatter
    if issubclass(cls, Formatter):
        format = args[0]
        cls = get_formatter(format)
        obj = cls(**kwargs)
    else:
        obj = cls(*args, **kwargs)
    return obj



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.shell.cmdshell import CmdShell
from ovirtcli.utils.typehelper import TypeHelper
from ovirtcli.utils.autocompletionhelper import AutoCompletionHelper
from ovirtsdk.infrastructure import brokers
from ovirtcli.utils.methodhelper import MethodHelper


class ActionCmdShell(CmdShell):
    NAME = 'action'

    def __init__(self, context, parser):
        CmdShell.__init__(self, context, parser)
        self.complete_exceptions = ['delete', 'update']
        self.identifier_template = '--%s-identifier'

    def do_action(self, args):
        return self.context.execute_string(ActionCmdShell.NAME + ' ' + args + '\n')

    def __add_resource_specific_options(self, obj, specific_options, line, key=None):
        obj_type = TypeHelper.getDecoratorType(TypeHelper.to_singular(obj))
        gda = MethodHelper.get_documented_arguments
        gat = TypeHelper.get_actionable_types
        memeber = key if key is not None else obj
        method_args = []
        action = None

        if obj_type and hasattr(brokers, obj_type):
            action = self.__get_method_name_by_args(obj,
                                                    line.split(' '),
                                                    gat(expendNestedTypes=True),
                                                    line)
            if action:
                obj_typ_ref = getattr(brokers, obj_type)
                if obj_typ_ref and hasattr(obj_typ_ref, action):
                    method_args = gda(method_ref=getattr(obj_typ_ref, action),
                                      as_params_collection=True,
                                      spilt_or=True)
                    if method_args:
                        specific_options[memeber] = method_args

    def __get_resource_actions(self, obj, args, line):
        gom = MethodHelper.get_object_methods
        gdt = TypeHelper.getDecoratorType
        obj_type = gdt(obj)
        actions = []

        if obj_type:
            #add base resource actions
            actions = gom(getattr(brokers, obj_type),
                          exceptions=self.complete_exceptions)

            #add sub-resources actions
            if  args.has_key(obj) and args[obj]:
                self.__add_sub_resourse_actions(obj, args, line, actions)

        return actions


    def __get_method_name_by_args(self, obj, spl, args, line):
        actions = self.__get_resource_actions(obj, args, line)

        for item in spl:
            if item in actions: return item

    def __add_sub_resourse_actions(self, obj, args, line, actions):
        gdt = TypeHelper.getDecoratorType
        gom = MethodHelper.get_object_methods

        if obj in args.keys():
            for cons in args[obj].split(', '):
                spl = line.strip().split(' ')
                if cons != 'None':
                    spl.append(self.identifier_template % cons)
                    base = self._resolve_base(spl[1:])
                    if base:
                        obj_type = gdt(TypeHelper.to_singular(base))
                        actions.extend(gom(getattr(brokers, obj_type),
                                           exceptions=self.complete_exceptions))


    def __add_resource_specific_arguments(self, obj, specific_options, line, key=None):
        gdt = TypeHelper.getDecoratorType
        obj_type = gdt(TypeHelper.to_singular(obj))
        args = TypeHelper.get_actionable_types(expendNestedTypes=True)
        gom = MethodHelper.get_object_methods
        actions = []

        method = self.__get_method_name_by_args(obj, line.split(' '), args, line)

        # collect resource args if yet not used
        if not method and obj_type and hasattr(brokers, obj_type):
            #add base resource actions
            actions = gom(getattr(brokers, obj_type),
                          exceptions=self.complete_exceptions)

            if args.has_key(obj) and args[obj]:
                #add sub-resources actions
                self.__add_sub_resourse_actions(obj, args, line, actions)

        specific_options[obj if key == None else key] = actions


    def __get_action_args(self, line):
        args = TypeHelper.get_actionable_types(expendNestedTypes=True)
        spl = line.rstrip().split(' ')
        gom = MethodHelper.get_object_methods
        gdt = TypeHelper.getDecoratorType

        if len(spl) >= 3:
            new_cons = ''
            obj = spl[1].strip()

            #top level resource in possibilities
            if args.has_key(obj) and args[obj] and args[obj].find('None') != -1:
                act = spl[3].strip() if len(spl) > 3 \
                                     else spl[-1].strip()
            #possibilities refer only to the sub-resource
            else:
                for item in spl:
                    if item.endswith('-identifier'):
                        args[obj] = new_cons
                        return args
                return args

            for cons in args[obj].split(', '):
                if cons == 'None':
                    obj_type = gdt(obj)
                    if not obj_type: continue
                    actions = gom(getattr(brokers, obj_type),
                                  exceptions=self.complete_exceptions)
                    if actions and act in actions:
                        new_cons += 'None, '
                else:
                    tmp_spl = spl[:]
                    tmp_spl.append(self.identifier_template % cons)
                    base = self._resolve_base(tmp_spl[1:])
                    if base:
                        obj_type = gdt(TypeHelper.to_singular(base))
                        if not obj_type: continue
                        actions = gom(getattr(brokers, obj_type),
                                      exceptions=self.complete_exceptions)
                        if actions and act in actions:
                            new_cons += cons + ', '

            args[obj] = new_cons[:len(new_cons) - 2] if len(new_cons) > 3 \
                                                     else ''

        return args

    def complete_action(self, text, line, begidx, endidx):
        args = self.__get_action_args(line)

        specific_options = self.get_resource_specific_options(args,
                                                              line,
                                                              callback=self.__add_resource_specific_options)

        specific_arguments = self.get_resource_specific_options(args,
                                                                line,
                                                                callback=self.__add_resource_specific_arguments)

        return AutoCompletionHelper.complete(line,
                                             text,
                                             args,
                                             specific_options=specific_options,
                                             specific_arguments=specific_arguments)

    def is_action_argument(self, line, key):
        args = self.__get_action_args(line)
        if key in args:
            return True
        specific_options = self.get_resource_specific_options(args,
                                                              line,
                                                              callback=self.__add_resource_specific_options)
        for arg_key in specific_options.keys():
             if key in specific_options[arg_key]:
                 return True
        return False




#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


class PromptMode():
    Original, Multiline, Disconnected, Connected, Unauthorized, Default = range(6)

    def __init__(self, Type):
        self.value = Type

    def __str__(self):
        if self.value == PromptMode.Original:
            return 'Original'
        if self.value == PromptMode.Multiline:
            return 'Multiline'
        if self.value == PromptMode.Disconnected:
            return 'Disconnected'
        if self.value == PromptMode.Connected:
            return 'Connected'
        if self.value == PromptMode.Unauthorized:
            return 'Unauthorized'
        if self.value == PromptMode.Default:
            return 'Default'

    def __eq__(self, y):
        return self.value == y.value



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from ovirtcli.shell.historycmdshell import HistoryCmdShell
from ovirtcli.command.command import OvirtCommand


class HistoryCommand(OvirtCommand):

    name = 'history'
    description = 'displaying commands history'
    args_check = (0, 1)
    valid_options = [ (
          '--' + item, int
         )
         for item in HistoryCmdShell.OPTIONS
    ]
    helptext = """\
        == Usage ==

        history [indx]

        or

        history --last n

        or

        history --first n

        or

        recursive search by ctrl+r

        == Description ==

        Displaying commands history

        == Examples ==

        - ctrl+r

        - history

        - history 1

        - history --last 3

        - history --first 3

        """

    def execute(self):
        args = self.arguments
        options = self.options

        if len(args) > 0:
            self.__printHistoryAtIndex(args)
        else:
            if len(options) > 0:
                self.__printPartialHistory(options)
            else:
                self.__printAllHistory()

    def __printAllHistory(self):
        context = self.context
        i = 0
        history = context.history.list()
        if history:
            self.write('')
            for item in history:
                self.__printHistoryItemAtIndex(i + 1, item)
                i += 1
            self.write('')

    def __printHistoryAtIndex(self, args):
        context = self.context
        indx = args[0]
        try:
            slide = int(indx)
            h_item = context.history.get(slide)
            self.write('')
            self.__printHistoryItemAtIndex(slide, h_item)
            self.write('')
        except Exception, e:
            self.error(str(e))

    def __printPartialHistory(self, options):
        context = self.context

        key = options.keys()[0]
        prop = key.replace('--', '')
        val = options[key]

        history_len = context.history.length()
        if history_len:
            history_len -= 1
            slide_len = int(val)
            try:
                if prop == 'last':
                    self.__printHistoryBetweenIndexes(max(0, history_len - slide_len), history_len)
                elif prop == 'first':
                    self.__printHistoryBetweenIndexes(0, min(slide_len, history_len))
            except Exception, e:
                self.error(str(e))

    def __printHistoryBetweenIndexes(self, from_index, to_index):
        context = self.context
        i = from_index
        history = context.history.listBetweenIndexes(from_index + 1, to_index + 1)
        if history:
            self.write('')
            for item in history:
                self.__printHistoryItemAtIndex(i + 1, item)
                i += 1
            self.write('')

    def __printHistoryItemAtIndex(self, slide, h_item):
        hformat = '[%d] %s'
        if h_item:
            self.write(hformat % (slide , str(h_item)))




#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import sys
import textwrap
from optparse import OptionParser, BadOptionError, AmbiguousOptionError
from ovirtcli.infrastructure import settings


class OvirtCliOptionParser(OptionParser):

    usage = '%prog [options]\n       %prog [options] command...'
    description = textwrap.dedent("""\
        This program is a command-line interface to %s Virtualization.
        """ % settings.OvirtCliSettings.PRODUCT)

    def __init__(self):

        OptionParser.__init__(self, usage=self.usage,
                              description=self.description)
        self.add_option('-d', '--debug', action='store_true',
                        help='enable debugging')
        self.add_option('-l', '--url',
                        help='specifies the API entry point URL (http[s]://server[:port]/ovirt-engine/api)')
        self.add_option('-u', '--username', help='connect as this user')
        self.add_option('-K', '--key-file', help='specify client PEM key-file')
        self.add_option('-C', '--cert-file', help='specify client PEM cert-file')
        self.add_option('-A', '--ca-file', help='specify server CA cert-file')
        self.add_option('-I', '--insecure', help='allow connecting to SSL sites without CA certificate',
                        action='store_true')
        self.add_option('-r', '--renew-session', help='automatically renew session upon expiration',
                        action='store_true')
        self.add_option('-D', '--dont-validate-cert-chain', help='do not validate server CA certificate',
                        action='store_true')
        self.add_option('-F', '--filter', help='enables user permission based filtering',
                        action='store_true')
        self.add_option('-P', '--port', help='specify port')
        self.add_option('-T', '--timeout', help='specify request timeout')
        self.add_option('-S', '--session-timeout', help='specify authentication session timeout in minutes (positive number)')
        self.add_option('-c', '--connect', action='store_true',
                        help='automatically connect')
        self.add_option('-f', '--file', metavar='FILE',
                        help='read commands from FILE instead of stdin')
        self.add_option('-e', '--extended-prompt', action='store_true',
                        help='display extra information in the prompt')
        self.add_option('-E', '--execute-command',
                        help='execute command and print the output to STDIO')
        self.disable_interspersed_args()

        # list of hidden app. options (long format)
        self.app_options = ['--password', '-p']

    def exit(self, status=0, msg=None):
        self.values._exit = True
        if msg: print (msg + 'see help for more details.\n')
        sys.exit(status)

    def _match_long_opt(self, opt):
        """_match_long_opt(opt : string) -> string

        Determine which long option string 'opt' matches, ie. which one
        it is an unambiguous abbrevation for.  Raises BadOptionError if
        'opt' doesn't unambiguously match any long option string.
        """
        return self._match_abbrev(opt, self._long_opt)

    def _match_abbrev(self, s, wordmap):
        """_match_abbrev(s : string, wordmap : {string : Option}) -> string

        Return the string key in 'wordmap' for which 's' is an unambiguous
        abbreviation.  If 's' is found to be ambiguous or doesn't match any of
        'words', raise BadOptionError.
        """

        # Is there an exact match?
        if s in wordmap:
            return s
        else:
            # Isolate all words with s as a prefix.
            option_keys = wordmap.keys()
            for item in self.app_options:
                if item not in  option_keys:
                    option_keys.append(item)
            possibilities = [word for word in option_keys
                             if word.startswith(s)]
            # No exact match, so there had better be just one possibility.
            if len(possibilities) == 1:
                return possibilities[0]
            elif not possibilities:
                raise BadOptionError(s)
            else:
                # More than one possible completion: ambiguous prefix.
                possibilities.sort()
                raise AmbiguousOptionError(s, possibilities)

    def _process_long_opt(self, rargs, values):
        arg = rargs.pop(0)

        # Value explicitly attached to arg?  Pretend it's the next
        # argument.
        if "=" in arg:
            (opt, next_arg) = arg.split("=", 1)
            rargs.insert(0, next_arg)
            had_explicit_value = True
        else:
            opt = arg
            had_explicit_value = False

        opt = self._match_long_opt(opt)
        if opt not in self._long_opt.keys() and opt in self.app_options:
            # This is app. option (long format)
            self.add_option('', opt, help='private app. option')
        option = self._long_opt[opt]
        if option.takes_value():
            nargs = option.nargs
            if len(rargs) < nargs:
                if nargs == 1:
                    self.error(_("%s option requires an argument") % opt)
                else:
                    self.error(_("%s option requires %d arguments")
                               % (opt, nargs))
            elif nargs == 1:
                value = rargs.pop(0)
            else:
                value = tuple(rargs[0:nargs])
                del rargs[0:nargs]

        elif had_explicit_value:
            self.error(_("%s option does not take a value") % opt)

        else:
            value = None

        option.process(opt, value, values, self)



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from cli import compat


class Warning(Exception):  # @ReservedAssignment
    """Base class for python-cli warnings."""

    def __init__(self, message=None, **kwargs):
        if message is None: message = self.__doc__
        message = message + '\n' if not message.endswith('\n') \
                                 else message
        compat.super(Warning, self).__init__(message)

        for key in kwargs:
            setattr(self, key, kwargs[key])

class GenericWarning(Warning):
    """Generic warning."""




#
# Copyright (c) 2013 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.state.dfsaevent import DFSAEvent
from ovirtcli.state.dfsastate import DFSAState
from ovirtcli.annotations.requires import Requires
from ovirtcli.events.event import Event
from ovirtcli.meta.singleton import Singleton

from cli.error import StateError, UnknownEventError

class FiniteStateMachine(object):
    '''
    The Deterministic Finite-State Automata (DFSA)

    =========== state maintains logic ===========

    A=(A, Q, l, q0, F)

    A is alphabetic
                                 _
    Q is a finite set of states |_|
                                      _      _
    l is a state transition function |_| -> |_|
                             _
    q0 is a initial state ->|_|

                               =====
    F is a set of final states || ||
                               =====

    L = {ab^n E A* : n >= 0}


    e.g:

        --b-------------------------
        |                          |
       ---       =====      ----   |
     ->|q0|-a--->||q1||-a-->|q2|<---
        --       =====      ----
                 |   ^      |   ^
                 b   |     a,b  |
                 |   |      |   |
                 -----      -----

    A = ({{a, b}, {q0,q1,q2},l,{q0}, {q1})

    e.g: l=(q0,a)=q1 l=(q1,b)=q1

    =========== usage ===========

    from ovirtcli.state.finitestatemachine import FiniteStateMachine
    from ovirtcli.state.dfsaevent import DFSAEvent
    from ovirtcli.state.dfsastate import DFSAState

    1. define the callback for events, with callback you can either
       perform onEvent actions or block the StateMachine till your
       action is accomplished.

        def onConnectCallback(self, **kwargs):
            print "connect callback with:\n%s\n\n" % kwargs['event']

        def onDisconnectCallback(self, **kwargs):
            print "disconnect callback:\n%s\n\n" % kwargs['event']

    2. define the StateMachine

    class Test(object):
        def run(self):
            sm = FiniteStateMachine( # define the StateMachine
                events=[ # define events
                    DFSAEvent( # define StateMachine event
                      name='disconnect', # event name
                      sources=[ # source states from which this event is eligible
                           DFSAState.Connected,
                           DFSAState.Unauthorized
                      ],
                      destination=DFSAState.Disconnected, # destination event state
                      callbacks=[self.onDisconnectCallback]), # callbacks to invoke after
                                                              # this events is triggered
                    DFSAEvent(
                      name='connect',
                      sources=[
                           DFSAState.Disconnected,
                           DFSAState.Unauthorized
                      ],
                      destination=DFSAState.Connected,
                      callbacks=[self.onConnectCallback]),
                    DFSAEvent(
                      name='unauthorized',
                      sources=[
                           DFSAState.Connected
                      ],
                      destination=DFSAState.Unauthorized,
                      callbacks=[]),
                ]
            )

    3. StateMachine has own events:

       - onBeforeApplyState: triggered Before ApplyState occurs
       - onAfterApplyState : triggered After ApplyState occurs
       - onStateChange,    : triggered on any StateChange
       - onBeforeEvent     : triggered Before Event processed
       - onAfterEvent      : triggered After Event processed
       - onCanMove         : triggered when CanMove() check is invoked

       you can register to them using event patter:

       sm.onBeforeEvent += OnBeforeEventListener()

       NOTE:

       - EventListener must to implement IListener interface
       - All events are provided with 'event' argument to maintain the state
         between the events, in some case extra data can be provided such as
         [source, destination, result, ...] depending on event's context.
       - callbacks can be registered later calling sm.add_callback(DFSAState.X, your_method)

    4. trigger events on StateMachine

            sm.connect() # 'connect' event
            # print sm; print "\n"
            sm.disconnect() 'disconnect' event
            # print sm; print "\n"
            sm.unauthorized() 'unauthorized' event
            # print sm; print "\n"
            ...

    Test().run()
    '''

    __metaclass__ = Singleton

#   @Requires([DFSAEvent], DFSAEvent)
#   TODO: support multi-parameters definition ^
    def __init__(self, events, inital_state=DFSAEvent(
                                      name='disconnected',
                                      sources=[],
                                      destination=DFSAState.DISCONNECTED,
                                      description='init'
                               )
        ):
        '''
        @param events: the list of DFSA events
        @param inital_state: the inital state of DFSA (optional)
        '''

        assert events != None

        self.__id = id(self)
        self.__current_state_obj = None
        self.__current_state = None
        self.__origin_state = None
        self.__origin_state_object = None
        self.__events = {}  # future use

        self.onBeforeApplyState = Event()
        self.onAfterApplyState = Event()
        self.onStateChange = Event()

        self.onBeforeEvent = Event()
        self.onAfterEvent = Event()

        self.onCanMove = Event()

        self.__register_events(events)
        self.__resolve_inital_state(inital_state)

#     @Requires(DFSAEvent)
    def __apply_state(self, event):
        """
        applying state

        @raise StateError: when event.destination state is
                           not applicable from the current_state
        """
        if self.can_move(event):
            self.onBeforeApplyState.fire(
                     event=event,
                     source=self.get_current_state(),
                     destination=event.get_destination()
            )
            self.onStateChange.fire(
                     event=event,
                     source=self.get_current_state(),
                     destination=event.get_destination()
            )

            old_state = self.get_current_state()

            self.__origin_state_object = self.__current_state_obj
            self.__current_state_obj = event
            self.__origin_state = self.__current_state
            self.__current_state = event.get_destination()

            self.onAfterApplyState.fire(
                     event=event,
                     source=old_state,
                     destination=self.get_current_state()
            )

        else:
            self.__raise_state_error(event)

    @Requires([DFSAEvent])
    def __register_events(self, events):
        """
        registers events

        @param events: the list of events to register
        """
        self.__events = {}
        for event in events:
            self.__do_add_event(event)

#     @Requires(DFSAEvent)
    def __produce_event_method(self, event):
        """
        produces event method

        @param eevent: event for which the method should
                       be procured
        """
        def event_method(**kwargs):
            if self.get_current_state() != event.get_destination():
                self.onBeforeEvent.fire(event=event)
                self.__apply_state(event)
                if event.get_callbacks():
                    for callback in event.get_callbacks():
                        # TODO: consider passing all **kwargs
                        callback(event=event)
                self.onAfterEvent.fire(event=event)
            else:
                return
        return event_method

#     @Requires(DFSAEvent)
    def __raise_state_error(self, event):
        """
        @raise StateError: when event.destination state is
                           not applicable from the current_state
        """
        raise StateError(
                 destination=event,
                 current=self.get_current_state()
        )

    def __raise_unknown_event(self, name):
        """
        @raise UnknownEventError: when is not registered
        """
        raise UnknownEventError(name=name)

#     @Requires(DFSAEvent)
    def __do_add_event(self, event):
        """
        registers new event in DFSM

        @param event: event to register
        """
        self.__events[event.get_name()] = event
        setattr(
            self,
            event.get_name(),
            self.__produce_event_method(event)
        )

#     @Requires(types.StringType)
    def __get_event(self, state):
        name = str(DFSAState(state)).lower()
        if name in self.__events.keys():
            return self.__events[name]
        self.__raise_unknown_event(name)

    def __str__(self):
        return 'FiniteStateMachine: %s, current state: %s' % (
               str(self.__id),
               DFSAState(self.get_current_state())
        )

    @Requires(DFSAEvent)
    def add_event(self, event):
        """
        adds or overrides DFSAEvent event to/in DFSA

        @param event: the DFSAEvent to add/override
        """
        self.__do_add_event(event)

    # @Requires(types.StringType, types.MethodType)
    def add_callback(self, state, callback):
        """
        adds new callback to event

        @param state: the state to which you would
                      like to add a callback
        @param callback: the method to register
        """
        self.__get_event(state) \
            .get_callbacks() \
            .append(callback)

    def get_current_state(self):
        """
        @return: the current State of DFSA
        """
        return self.__current_state

    def get_origin_state(self):
        """
        @return: the origin State of DFSA
        """
        return self.__origin_state

    def get_origin_state_event(self):
        """
        @return: the origin State of DFSA
        """
        return self.__origin_state_object

    @Requires(DFSAEvent)
    def can_move(self, event):
        """
        checks if DFSA can move to the given event.destination

        @param event: the destination DFSAEvent
        """
        if not self.__current_state_obj:
            result = True  # can happen during init only!
            # TODO: consider restricting this behavior
        else:
            result = self.__current_state in event.get_sources()

        self.onCanMove.fire(
                 event=event,
                 source=self.get_current_state(),
                 destination=event.get_destination(),
                 result=result
        )

        return result

    def rollback(self):
        """
        performs a rollback to the origin state
        """
        self.__apply_state(self.get_origin_state_event())

    def __resolve_inital_state(self, inital_state):
        """
        resolves initial state of DFSA

        if user has defined same state as default one and DFSA can move to it -
        it will be used, otherwise default state will be set as a initial.
        """
        if self.__events.has_key(inital_state.get_name()) and \
           self.can_move(inital_state):
            self.__apply_state(
                   self.__events.get(
                         inital_state.get_name()
                   )
            )
        else:
            self.__apply_state(inital_state)



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from fnmatch import fnmatch
from cli.error import CommandError
from ovirtcli.format.help import Help
from ovirtcli.utils.methodhelper import MethodHelper
from ovirtcli.utils.multivaluedict import MultiValueDict
from cli.warning import GenericWarning


class Command(object):
    """Base class for all commands."""

    name = None
    aliases = ()
    description = None
    helptext = None
    args_check = 0
    valid_options = [ ('--help', None) ]

    def __init__(self, arguments, options):
        """Constructor."""
        self.arguments = arguments
        self.options = MultiValueDict(options)
        if '--help' in self.options:
            self.mode = 'show_help'
        else:
            self.mode = 'execute'
        if self.mode == 'execute':
            self.check_arguments()
            self.check_options()

    def check_arguments(self):
        """Check arguments for validity."""
        args = self.arguments
        if self.args_check is None:
            argsok = True
        elif isinstance(self.args_check, int):
            argsok = self.args_check == len(args)
        elif callable(self.args_check):
            argsok = self.args_check(args)
        elif isinstance(self.args_check, tuple):
            argsok = len(args) in self.args_check
        if not argsok:
            raise SyntaxError, 'wrong number of arguments, try \'help %s\' for help.' % self.name

    def check_options(self):
        """Check options for validity."""
        for key in self.options:
            for name, validator in self.valid_options:
                if not fnmatch(key, name):
                    continue
                value = self.options[key]
                if validator is None:
                    if value is not None:
                        self.error('option %s takes no argument (provided: %s)' %
                                   (key, value))
                else:
                    try:
                        value = validator(value)
                    except ValueError:
                        self.error('could not validate option %s (provided: %s)' %
                                   (key, value))
                break
            else:
                self.error('unknown option: %s' % key)

    def show_help(self):
        """Show help for the current command."""
        subst = {}
        subst['command'] = self.context.command.name
        commands = self.get_commands()
        subst['commands'] = self.format_list(commands, sort=True)
        statuses = self.get_statuses()
        subst['statuses'] = self.format_list(statuses, sort=True)
        helptext = self.format_help(self.helptext, subst)
        stdout = self.context.terminal.stdout
        stdout.write(helptext)

    def run(self, context):
        """Entry point. This either executes a command or shows help."""
        self.context = context
        if self.mode == 'show_help':
            self.show_help()
        elif self.mode == 'execute':
            self.execute()

    def execute(self, context):
        """Override this method in a subclass."""

    def warning(self, message, **kwargs):
        raise GenericWarning(message, **kwargs)

    def write(self, message):
        self.context.terminal.stdout.write(message + '\n')

    def error(self, message, cls=None, **kwargs):
        """Raise an error. This function does not return. If the user is in
        active mode, an error message is displayed and he will be allowed to
        enter another command."""

        print ''

        if cls is None:
            cls = CommandError
        elif not issubclass(cls, CommandError):
            raise TypeError, 'Expecting a CommandError subclass'
        raise cls(message, **kwargs)

    def format_list(self, lst, bullet='*', indent=0, sort=False):
        """Format a list of items, to be used with format_help()."""
        formatted = []
        for elem in lst:
            line = ' ' * indent
            if bullet and elem.find(MethodHelper.NON_ARG_TEMPLATE) == -1:
                line += bullet + ' '
            line += elem.replace(MethodHelper.NON_ARG_TEMPLATE, '')
            formatted.append(line)
        if sort: formatted = sorted(formatted)
        formatted = '\n'.join(formatted)
        return formatted

    def format_map(self, mp, bullet='*', indent=0, sort=True):
        """Format a list of items, to be used with format_help()."""
        formatted = []
        for elem in mp.keys():
            if sort and mp[elem] and type(mp[elem]) == list: mp[elem] = sorted(mp[elem])
            line = ' ' * indent
            if bullet:
                line += bullet + ' '
            if mp[elem] != None:
                line += elem + ' (contexts: ' + str(mp[elem]) + ')'
            else:
                line += elem
            formatted.append(line)
        if sort: formatted = sorted(formatted)
        formatted = '\n'.join(formatted)
        return formatted

    def format_help(self, text, subst):
        return Help.format(text, subst)

    def get_statuses(self):
        """Return a list of all exist statuses that are defined."""
        result = []
        for sym in dir(self.context):
            if not sym.isupper():
                continue
            value = getattr(self.context, sym)
            if not isinstance(value, int):
                continue
            result.append('%03d (%s)' % (value, sym))
        return result

    def get_commands(self):
        """Return a list of all available commands."""
        commands = []
        for cmd in self.context.commands:
            commands.append('%-16s %s' % (cmd.name, cmd.description))
        return commands



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import sys
import os
import cmd
import signal
import readline
import re

from ovirtcli.shell.actioncmdshell import ActionCmdShell
from ovirtcli.shell.connectcmdshell import ConnectCmdShell
from ovirtcli.shell.showcmdshell import ShowCmdShell
from ovirtcli.shell.listcmdshell import ListCmdShell
from ovirtcli.shell.updatecmdshell import UpdateCmdShell
from ovirtcli.shell.removecmdshell import RemoveCmdShell
from ovirtcli.shell.addcmdshell import AddCmdShell
from ovirtcli.shell.disconnectcmdshell import DisconnectCmdShell
from ovirtcli.shell.consolecmdshell import ConsoleCmdShell
from ovirtcli.shell.pingcmdshell import PingCmdShell
from ovirtcli.shell.statuscmdshell import StatusCmdShell
from ovirtcli.shell.clearcmdshell import ClearCmdShell
from ovirtcli.shell.summarycmdshell import SummaryCmdShell
from ovirtcli.shell.filecmdshell import FileCmdShell
from ovirtcli.shell.historycmdshell import HistoryCmdShell
from ovirtcli.shell.infocmdshell import InfoCmdShell
from ovirtcli.shell.capabilitiescmdshell import CapabilitiesCmdShell

from cli.command.help import HelpCommand
from cli.messages import Messages

from ovirtcli.events.event import Event
from ovirtcli.listeners.errorlistener import ErrorListener
from ovirtcli.infrastructure.settings import OvirtCliSettings
from ovirtcli.shell.promptmode import PromptMode
from cli.error import CommandError

from ovirtcli.state.statemachine import StateMachine
from ovirtcli.state.dfsastate import DFSAState
from ovirtcli.shell.promptmanager import PromptManager

class EngineShell(cmd.Cmd, ConnectCmdShell, ActionCmdShell, \
                  ShowCmdShell, ListCmdShell, UpdateCmdShell, \
                  RemoveCmdShell, AddCmdShell, DisconnectCmdShell, \
                  ConsoleCmdShell, PingCmdShell, StatusCmdShell, \
                  ClearCmdShell, FileCmdShell, HistoryCmdShell, \
                  InfoCmdShell, SummaryCmdShell, CapabilitiesCmdShell):
    OFF_LINE_CONTENT = [ConnectCmdShell.NAME, HelpCommand.name, 'exit', "EOF"]

    ############################# INIT #################################

    def __init__(self, context, parser, completekey='tab', stdin=None, stdout=None):
        cmd.Cmd.__init__(self, completekey=completekey, stdin=stdin, stdout=stdout)
        ConnectCmdShell.__init__(self, context, parser)
        ActionCmdShell.__init__(self, context, parser)
        ShowCmdShell.__init__(self, context, parser)
        ListCmdShell.__init__(self, context, parser)
        UpdateCmdShell.__init__(self, context, parser)
        RemoveCmdShell.__init__(self, context, parser)
        AddCmdShell.__init__(self, context, parser)
        DisconnectCmdShell.__init__(self, context, parser)
        ConsoleCmdShell.__init__(self, context, parser)
        PingCmdShell.__init__(self, context, parser)
        StatusCmdShell.__init__(self, context, parser)
        ClearCmdShell.__init__(self, context, parser)
        FileCmdShell.__init__(self, context, parser)
        HistoryCmdShell.__init__(self, context, parser)
        InfoCmdShell.__init__(self, context, parser)
        SummaryCmdShell.__init__(self, context, parser)
        CapabilitiesCmdShell.__init__(self, context, parser)

        self.__last_output = ''
        self.__input_buffer = ''
        self.__last_status = -1

        self.onError = Event()  # triggered when error occurs
        self.onInit = Event()  # triggered on init()
        self.onExit = Event()  # triggered on exit
        self.onPromptChange = Event()  # triggered onPromptChange
        self.onSigInt = Event()  # triggered on SigInt fault

        self.__prompt_manager = PromptManager(self)

        self.__register_sys_listeners()
        self.__register_dfsm_callbacks()
        self.__init_promt()

        cmd.Cmd.doc_header = self.context.settings.get('ovirt-shell:commands')
        cmd.Cmd.undoc_header = self.context.settings.get('ovirt-shell:misc_commands')
        cmd.Cmd.intro = self.__get_intro()

        readline.set_completer_delims(' ')
        signal.signal(signal.SIGINT, self.__handler)

        self.onInit.fire()

    ############################ SHELL ##################################

    def cmdloop(self, intro=None, clear=True):
        try:
            if clear: self.do_clear('')
            return cmd.Cmd.cmdloop(self, intro)
        except KeyboardInterrupt:
            return self.cmdloop(intro="^C", clear=False)
        except Exception, e:
            self._error(str(e))
            return self.cmdloop(intro, clear=False)

    def print_line(self, line):
        print self.prompt + line

    def emptyline(self, no_prompt=False):
        if no_prompt:
            print ''
        else:
            print self.prompt

    def onecmd(self, s):
        if not s.startswith('#'):
            command = s.split(' ')[0]
            if command == '' and not self.__input_buffer:
                pass
            elif self.context.connection == None and \
                    command.strip() not in EngineShell.OFF_LINE_CONTENT:
                self._error(Messages.Error.INVALID_COMMAND % command)
            else:
                try:
                    if s.endswith('\\') and s != 'EOF':
                        self.__set_prompt(mode=PromptMode.Multiline)
                        self.__input_buffer += ' ' + s.replace('\\', '') \
                        if not s.startswith(' ') and self.__input_buffer != ''\
                        else s.replace('\\', '')
                        return
                    elif self.__input_buffer != '' and s != 'EOF':
                        self.__input_buffer += ' ' + s \
                        if not s.startswith(' ') else s
                        s = self.__input_buffer
                        self.__input_buffer = ''
                        self.__set_prompt(mode=PromptMode.Original)
                    return cmd.Cmd.onecmd(self, s)
                finally:
                    if self.context.status == \
                       self.context.SYNTAX_ERROR \
                       or \
                       self.context.status == \
                       self.context.COMMAND_ERROR \
                       or \
                       self.context.status == \
                       self.context.UNKNOWN_ERROR:
                        self.onError.fire()
                    elif self.context.status == \
                       self.context.COMMUNICATION_ERROR:
                        self.__last_status = self.context.status
                        self.onError.fire()
                        StateMachine.communication_error()  # @UndefinedVariable
                    elif self.context.status == \
                       self.context.AUTHENTICATION_ERROR:
                        self.__last_status = self.context.status
                        self.onError.fire()
                        StateMachine.unauthorized()  # @UndefinedVariable
                    elif self.__last_status <> -1 and \
                         (
                            (
                                self.__last_status == self.context.COMMUNICATION_ERROR
                                and
                                StateMachine.get_current_state() != DFSAState.COMMUNICATION_ERROR
                            )
                          or \
                            (
                                self.__last_status == self.context.AUTHENTICATION_ERROR
                                and
                                StateMachine.get_current_state() != DFSAState.UNAUTHORIZED
                            )
                         ):
                        self.__set_prompt(
                          mode=PromptMode.Original
                        )
                        self.__last_status = -1

    ########################### SYSTEM #################################
    def __get_intro(self):
        """
        @return: the shell intro
        """
        return self.__produce_screen_adapted_message(
                                      text=OvirtCliSettings.INTRO,
                                      border="+"
        )

    def __produce_screen_adapted_message(self, text, border, newline="\n\n"):
        """
        produces (pretty) screen width adapted message
        
        @param text: text to prin
        @param border: border to use (default is '+')
        @param newline: new line separator
        """
        return self.context.formatter.format_terminal(
                          text=text,
                          border=border,
                          termwidth=self.context.terminal._get_width(),
                          newline=newline
        )

    def __init_promt(self):
        self.__set_prompt(mode=PromptMode.Disconnected)

    def __register_dfsm_callbacks(self):
        """
        registers StateMachine events callbacks
        """
        StateMachine.add_callback(
                      DFSAState.UNAUTHORIZED,
                      self.__on_unauthorized_callback
        )
        StateMachine.add_callback(
                      DFSAState.COMMUNICATION_ERROR,
                      self.__on_communication_error_callback
        )
        StateMachine.add_callback(
                      DFSAState.DISCONNECTED,
                      self.__on_disconnected_callback
        )
        StateMachine.add_callback(
                      DFSAState.CONNECTED,
                      self.__on_connected_callback
        )
        StateMachine.add_callback(
                      DFSAState.EXITING,
                      self.__on_exiting_callback
        )

    def __on_unauthorized_callback(self, **kwargs):
        """
        triggered when StateMachine.UNAUTHORIZED state is acquired
        """
        self.__set_prompt(
            mode=PromptMode.Unauthorized
        )

    def __on_communication_error_callback(self, **kwargs):
        """
        triggered when StateMachine.COMMUNICATION_ERROR state is acquired
        """
        self.__set_prompt(
            mode=PromptMode.Disconnected
        )

    def __on_connected_callback(self, **kwargs):
        """
        triggered when StateMachine.CONNECTED state is acquired
        """
        self.context.history.enable()
        if not self.context.settings.get('ovirt-shell:execute_command'):
            self._print(
               self.__produce_screen_adapted_message(
                             text=OvirtCliSettings.CONNECTED_TEMPLATE,
                             border='=',
                             newline='\n'
               ) % \
               self.context.settings.get('ovirt-shell:version')
            )

        self.__set_prompt(
            mode=PromptMode.Connected
        )

    def __on_disconnected_callback(self, **kwargs):
        """
        triggered when StateMachine.DISCONNECTED state is acquired
        """
        self.context.history.disable()
        if not self.context.settings.get('ovirt-shell:execute_command'):
            self._print(
                "\n" + \
                self.__produce_screen_adapted_message(
                             text=OvirtCliSettings.DISCONNECTED_TEMPLATE,
                             border='=',
                             newline='\n'
               )
            )
        self.__set_prompt(
            mode=PromptMode.Disconnected
        )

    def __on_exiting_callback(self, **kwargs):
        """
        triggered when StateMachine.EXITING state is acquired
        """
        if StateMachine.get_origin_state() != DFSAState.DISCONNECTED \
           and \
           self.context.connection:
            self.do_disconnect('')
        else:
            self.do_echo("\n")  # break shell prompt in to bash shell

    def __register_sys_listeners(self):
        self.onError += ErrorListener(self)

    def __set_prompt(self, mode):
        self.__prompt_manager.set_prompt(mode)

    def __persist_cmd_options(self, opts):
        """
        Overrides config file options with cmdline's.
        """
        if opts:
            for k, v in opts.__dict__.items():
                if v:
                    if self.context.settings.has_key('ovirt-shell:' + k):
                        self.context.settings['ovirt-shell:' + k] = v
                    elif self.context.settings.has_key('cli:' + k):
                        self.context.settings['cli:' + k] = v

    def _get_last_status(self):
        return self.__last_status

    def onecmd_loop(self, s):
        opts, args = self.parser.parse_args()
        del args
        self.__persist_cmd_options(opts)
        if opts.connect or self.context.settings.get('cli:autoconnect'):
            self.context._collect_connection_data()
            if not self.context.settings.get('ovirt-shell:execute_command'):
                self.do_clear('')
            self.do_connect(s)
            if opts.file:
                self.do_file(opts.file)
                self.do_exit('')
            elif opts.execute_command:
                self.__execute_command(opts.execute_command)
                self.do_exit('')
            else:
                self.cmdloop(clear=False)
        else:
            self.cmdloop()

    def precmd(self, line):
        return cmd.Cmd.precmd(self, self.normalize(line))

    def normalize(self, line):
        normalized = line.lstrip()
        rg = re.compile('((?:[a-z][a-z]+))( )(-)(-)(help)', re.IGNORECASE | re.DOTALL)
        m = rg.search(normalized)
        if m:
            normalized = 'help ' + m.group(1)
        return normalized

    def parseline(self, line):
        ret = cmd.Cmd.parseline(self, line)
        return ret

    def completenames(self, text, *ignored):
        dotext = 'do_' + text
        res = [a[3:] for a in self.get_names() if a.startswith(dotext)]
        if res and len(res) == 1:
            res[0] = res[0] + ' '
        return res

    def complete(self, text, state):
        content = []
        line = readline.get_line_buffer().lstrip()

        if self.context.connection == None:
            if not line or len(line) == 0:
                content = EngineShell.OFF_LINE_CONTENT
            elif not line.split(' ')[0] in EngineShell.OFF_LINE_CONTENT or \
                 (len(line.split(' ')) > 1 and line.split(' ')[0] == HelpCommand.name):
                content = [ f
                            for f in EngineShell.OFF_LINE_CONTENT
                            if f.startswith(text)
                          ]
                if len(content) == 0: content = None

        return self.__do_complete(text, state, content=content)

    def __get_begidx(self, text):
        indx = 0
        i = 0
        for char in text:
            i += 1
            if char == ' ' and i != (indx + 1):
                indx = i
        return indx

    def __get_endidx(self, text):
        return len(text)

    def __do_complete(self, text, state, content=[]):
        """Return the next possible completion for 'text'"""
        if text.startswith('--'):
            text = text.strip()[2:]
        if state == 0:
            self.current_cmd = None
            self.cmd_line = None
            self.has_cmd_and_entity = False
            if self.__input_buffer != '':
                if not self.__input_buffer.endswith(readline.get_line_buffer()):
                    origline = self.__input_buffer + (' ' + readline.get_line_buffer()) \
                    if not self.__input_buffer.endswith(' ') \
                    else self.__input_buffer + readline.get_line_buffer() + ' '
                else:
                    origline = self.__input_buffer + ' '
                line = origline.lstrip()
                stripped = len(origline) - len(line)
                begidx = self.__get_begidx(origline) - stripped
                endidx = self.__get_endidx(origline) - stripped
            else:
                origline = readline.get_line_buffer()
                line = origline.lstrip()
                stripped = len(origline) - len(line)
                begidx = readline.get_begidx() - stripped
                endidx = readline.get_endidx() - stripped

            if begidx > 0:
                cmd, args, foo = self.parseline(line)
                if cmd == '':
                    compfunc = self.completedefault
                else:
                    entity, args, foo = self.parseline(args)
                    if entity != None and args != None and args != '':
                        self.has_cmd_and_entity = True
                    try:
                        self.current_cmd = cmd
                        compfunc = getattr(self, 'complete_' + cmd)
                    except AttributeError:
                        compfunc = self.completedefault
            else:
                compfunc = self.completenames
            if content and len(content) > 0:
                self.completion_matches = content
            elif content == None:
                self.completion_matches = []
            else:
                self.cmd_line = line
                self.completion_matches = compfunc(text, line, begidx, endidx)
        try:
            if self.has_cmd_and_entity and text != '' and len(self.completion_matches) == 1:
                retval =  self.completion_matches[state]
                is_parameter = False
                if retval != None and self.current_cmd != None and self.cmd_line != None:
                    evalfunc = getattr(self, 'is_' + self.current_cmd + '_argument')
                    if evalfunc:
                        is_parameter = evalfunc(self.cmd_line, retval)
                if retval != None and is_parameter == True and not retval.startswith('--'):
                    return '--' + retval
            return self.completion_matches[state]
        except IndexError:
            return None

    def _error(self, msg):
        # this is a custom error, consumer responsibility
        # to filter out duplicate calls on event
        self.onError.fire()
        self.context._handle_exception(CommandError(msg))

    def _print(self, msg, file=sys.stdout):  # @ReservedAssignment
        """
        prints message to the output device (default stdout).

        @param msg: text to print
        @param file: output device
        """
        self.context._pint_text(msg, file)

    def __execute_command(self, execute):
        """
        executes command from the command line
        via -E/--execute-command

        @param execute: the command to execute
        """
        self.context.settings['cli:autopage'] = False
        for command in execute.split(';'):
            line = self.precmd(command)
            self.print_line(line)
            self.onecmd(line)

    def __handler(self, signum, frame):
        self.onSigInt.fire()
        raise KeyboardInterrupt

    ############################# COMMANDS #################################

    def do_EOF(self, line):
        """\
        == Usage ==

        Ctrl+D

        == Description ==

        Exists shell by /Ctrl+D/ sequence.

        == Examples ==

        Ctrl+D
        """
        self.onExit.fire()
        StateMachine.exiting()  # @UndefinedVariable
        return True

    def do_exit(self, args):
        """\
        == Usage ==

        exit

        == Description ==

        Exists shell by /exit/ command.

        == Examples ==

        exit
        """
        self.onExit.fire()
        StateMachine.exiting()  # @UndefinedVariable
        sys.exit(0)

    def do_help(self, args):
        """\
        == Usage ==

        help ...

        == Description ==

        Prints help by command.

        == Examples ==

        help show
        """
        if not args:
            self.context.execute_string('help\n')
        else:
            cmd = args.split(' ')[0]
            if self.context.connection == None and cmd not in EngineShell.OFF_LINE_CONTENT:
                self._error(Messages.Error.INVALID_COMMAND % cmd)
            else:
                if hasattr(self, 'do_' + cmd) and getattr(self, 'do_' + cmd).__doc__:
                    self.context.terminal.stdout.write('\n' + getattr(self, 'do_' + cmd).__doc__ + '\n')
                else:
                    return self.context.execute_string('help ' + args + '\n')

    def do_shell(self, line):
        """\
        == Usage ==

        shell ...

        == Description ==

        Runs a shell command ('!' can be used instead of 'shell' command).

        == Examples ==

        shell ls -la

        or

        ! ls -la
        """
        output = os.popen(line).read()
        print output
        self.__last_output = output

    def do_echo(self, line):
        """\
        == Usage ==

        echo

        == Description ==

        Prints the input, replacing '$out' with the output of the last shell command output

        == Examples ==

        echo $out

        or

        echo str
        """
        if self.__last_output:
            print line.replace('$out', self.__last_output)
        elif line:
            print line
        else: print self.prompt




from cli.command.command import Command
from cli.command.cd import CdCommand
from cli.command.clear import ClearCommand
from cli.command.exit import ExitCommand
from cli.command.help import HelpCommand
from cli.command.pwd import PwdCommand
from cli.command.save import SaveCommand
from cli.command.set import SetCommand
from cli.command.status import StatusCommand



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.format.format import Formatter
from ovirtcli.format.xml_ import XmlFormatter
from ovirtcli.format.text import TextFormatter


def get_formatter(format):
    """Return the formatter class for `format', or None if it doesn't exist."""
    for sym in globals():
        obj = globals()[sym]
        if isinstance(obj, type) and issubclass(obj, Formatter) \
                and obj.name == format:
            return obj



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


class ExecutionMode():
    SHELL, SCRIPT, DEFAULT, NOPAGING = range(4)

    def __init__(self, Type):
        self.value = Type

    def __str__(self):
        if self.value == ExecutionMode.SHELL:
            return 'SHELL'
        if self.value == ExecutionMode.SCRIPT:
            return 'SCRIPT'
        if self.value == ExecutionMode.DEFAULT:
            return 'DEFAULT'
        if self.value == ExecutionMode.NOPAGING:
            return 'NOPAGING'

    def __eq__(self, y):
        return self.value == y.value



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.command.command import OvirtCommand
from ovirtcli.utils.typehelper import TypeHelper
from cli.messages import Messages


class ShowCommand(OvirtCommand):

    name = 'show'
    description = 'show one object'
    args_check = lambda self, x: len(x) > 0
    valid_options = [ ('*', str) ]

    helptext = """\
        == Usage ==
    
        show <type> <id|name> [parent identifiers] [command options]

        == Description ==

        Retrieve an object and display information about it. The following
        arguments are required:

          * type        The type of object to retrieve
          * id          The object identifier
          * name        The object name

        Objects can be identified by their name and by their unique id.

        == Supported Help formats ==

        - This help will show all available attribute options for showing 
          resource
          
          * format      - help show type
          * example     - help show vm

        == Available Types ==

        The <type> parameter must be one of the following.

          $types

        == Object Identifiers ==

        Some objects can only exist inside other objects. For example, a disk
        can only exist in the content of a virtual machine. In this case, one
        or more object identifier opties needs to be provided to identify the
        containing object.

        An object identifier is an option of the form '--<type>id <id>'. This
        would identify an object with type <type> and id <id>. See the
        examples section below for a few examples.

        == Examples ==

        - This example shows information about the virtual machine "myvm"

          $ show vm myvm

        - This example shows information about the nic named 'nic1' of the 
          virtual machine "myvm"

          $ show nic nic1 --vm-identifier myvm

        == Return values ==

        This command will exit with one of the following statuses. To see the
        exit status of the last command, type 'status'.

          $statuses
        """

    helptext1 = """\
        == Usage ==

        - show <type>
            
        - show <type> <id> [parent identifiers] [command options]

        == Description ==

        Shows an object by type '$type'. See 'help show' for generic
        help on showing objects.

        == Attribute Options ==

        The following options are available for objects with type '$type':

          $options

        == Return values ==

        This command will exit with one of the following statuses. To see the
        exit status of the last command, type 'status'.

          $statuses
        """

    def execute(self):
        """Execute "show"."""
        args = self.arguments
        opts = self.options

        # Raise an error if object identifier xxx is not specified #855750
        # e.g:
        # show vm xxx
        # show disk xxx --vm-identifier yyy
        if len(args) < 2 and (
                          len(opts) == 0 or
                          (
                           len(opts) == 1
                           and
                           opts.keys()[0].endswith('-identifier')
                          )):
            self.error(
              Messages.Error.NO_IDENTIFIER % args[0]
            )

        types = self.get_singular_types(method='get')
        obj = self.get_object(
              typ=args[0],
              obj_id=args[1] if len(args) > 1
                             else None,
              base=self.resolve_base(opts),
              opts=opts,
              context_variants=types[args[0]]
        )

        if not (obj):
            self.error(
               Messages.Error.NO_SUCH_OBJECT %
               (args[0], args[1] if len(args) > 1
                                 else self.get_object_id(opts.values())
                                      if opts else '')
            )
        self.context.formatter.format(self.context, obj)

    def get_object_id(self, opts_values):
        """Get Object Identifier."""
        if len(opts_values) == 1:
            return opts_values[0]
        return opts_values

    def show_help(self):
        """Show help for "show"."""
        self.check_connection()
        args = self.arguments
        opts = self.options

        subst = {}
        types = self.get_singular_types(method='get')

        if not args or self.is_supported_type(types.keys(), args[0]):

            subst['types'] = self.format_map(types)

            statuses = self.get_statuses()
            subst['statuses'] = self.format_list(statuses)

            if len(args) == 1:
                helptext = self.helptext1
                params_list = self.get_options(
                       method='get',
                       resource=TypeHelper.to_singular(args[0]),
                       sub_resource=self.resolve_base(opts),
                       context_variants=types[args[0]]
                )
                subst['options'] = self.format_list(params_list)
                subst['type'] = args[0]
            elif len(args) == 2:
                helptext = self.helptext1

                subst['type'] = args[0]
                subst['id'] = args[1]
                base = self.resolve_base(opts)
                obj = self.get_object(args[0], args[1], base)
                if obj is None:
                    self.error(
                       Messages.Error.NO_SUCH_OBJECT % (args[0], args[1])
                    )

                params_list = self.get_options(
                       method='get',
                       resource=obj,
                       sub_resource=base,
                       context_variants=types[args[0]]
                )
                subst['options'] = self.format_list(params_list)

            else:
                helptext = self.helptext

            helptext = self.format_help(helptext, subst)
            self.write(helptext)



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from cli.command import *
from cli.context import ExecutionContext

from ovirtcli.infrastructure.settings import OvirtCliSettings
from ovirtcli.command import *
from ovirtcli.format import *

from ovirtcli.infrastructure.object import create
import pkg_resources
from ovirtcli.command.info import InfoCommand
from ovirtcli.command.summary import SummaryCommand
from ovirtcli.command.capabilities import CapabilitiesCommand
from ovirtcli.infrastructure.historymanager import HistoryManager


class OvirtCliExecutionContext(ExecutionContext):

    Settings = OvirtCliSettings

    name = '%s-shell' % OvirtCliSettings.PRODUCT.lower()

    REMOTE_ERROR = 10
    NOT_FOUND = 11

    SDK_MODULE_NAME = "ovirt-engine-sdk-python"
    CLI_MODULE_NAME = "ovirt-shell"
    DEFAULT_VERSION = (0, 0, 0, 0)

    def __init__(self, args=None, option_parser=None):
        super(OvirtCliExecutionContext, self).__init__(args=args, option_parser=option_parser)
        self.connection = None
        self.url = None
        self.formatter = create(Formatter, self.settings['ovirt-shell:output_format'])
        self.settings.add_callback('cli:verbosity', self._set_verbosity)
        self.settings.add_callback('ovirt-shell:output_format', self._set_formatter)
        self.product_info = None
        self.sdk_version, self.cli_version, self.backend_version = self.__get_version_info()
        self.history = HistoryManager()
        self.__capabilities = None

    def set_connection(self, connection, url=None):
        self.connection = connection
        self.url = url
        self.__update_backend_metadata()

    def get_capabilities(self):
        """
        Fetches system capabilities

        @return: Capabilities
        """

        # cache capabilities cause it won't change
        # during application lifetime and deserializing
        # this object is pretty expensive.
        if not self.__capabilities:
            self.__capabilities = self.connection.capabilities.list()
        return self.__capabilities

    def __get_version_info(self):
        SNAPSHOT_SUFFIX = '-SNAPSHOT'

        try:
            sdk_version = pkg_resources.parse_version(
                              pkg_resources.get_distribution(self.SDK_MODULE_NAME)
                                           .version.replace(SNAPSHOT_SUFFIX, ''))

            cli_version = pkg_resources.parse_version(
                              pkg_resources.get_distribution(self.CLI_MODULE_NAME)
                                           .version.replace(SNAPSHOT_SUFFIX, ''))

            return sdk_version, cli_version, self.DEFAULT_VERSION
        except:
            return self.DEFAULT_VERSION, self.DEFAULT_VERSION, self.DEFAULT_VERSION

    def _set_verbosity(self, key, value):
        if self.connection is None:
            return
        if value:
            level = 10
        else:
            level = 1
        self.connection.verbosity = level

    def _set_formatter(self, key, value):
        self.formatter = create(Formatter, value)

    def _handle_exception(self, e):
        super(OvirtCliExecutionContext, self)._handle_exception(e)

    def setup_commands(self):
#        self.add_command(SetCommand)
#        self.add_command(SaveCommand)
#        self.add_command(CdCommand)
#        self.add_command(PwdCommand)
        self.add_command(ClearCommand)
        self.add_command(ExitCommand)
        self.add_command(ActionCommand)
        self.add_command(AddCommand)
        self.add_command(ConnectCommand)
        self.add_command(ConsoleCommand)
        self.add_command(RemoveCommand)
        self.add_command(DisconnectCommand)
        self.add_command(HelpCommand)
        self.add_command(ListCommand)
        self.add_command(PingCommand)
        self.add_command(ShowCommand)
        self.add_command(StatusCommand)
        self.add_command(UpdateCommand)
        self.add_command(InfoCommand)
        self.add_command(HistoryCommand)
        self.add_command(SummaryCommand)
        self.add_command(CapabilitiesCommand)

    def __update_backend_metadata(self):
        """Return a dict with prompt variables."""

        version = self.__get_backend_version()
        self.settings['ovirt-shell:version'] = version
        self.backend_version = pkg_resources.parse_version(version)

        return {'version':version}

    def __get_backend_version(self):
        backend_version = self.DEFAULT_VERSION
        if self.connection:
            self.product_info = self.connection.get_product_info()
            if self.product_info:
                version = self.product_info.version
                backend_version = (version.major, version.minor,
                                   version.build_, version.revision)
        return '%s.%s.%s.%s' % backend_version

    def _read_command(self):
        return super(OvirtCliExecutionContext, self)._read_command()

    def _clean_settings(self):
        self.settings['ovirt-shell:url'] = ''
        self.settings['ovirt-shell:username'] = ''
        self.settings['ovirt-shell:password'] = ''
        self.settings['ovirt-shell:key_file'] = None
        self.settings['ovirt-shell:cert_file'] = None
        self.settings['ovirt-shell:ca_file'] = None
        self.settings['ovirt-shell:insecure'] = False
        self.settings['ovirt-shell:port'] = -1
        self.settings['ovirt-shell:timeout'] = -1
        self.settings['ovirt-shell:session_timeout'] = -1



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import inspect
from ovirtsdk.utils.ordereddict import OrderedDict

class MethodHelper():

    NON_ARG_TEMPLATE = ':not-arg'

    @staticmethod
    def getMethodArgs(module, cls, method, get_varargs=False, get_keywords=False, drop_self=False, exclude=['context']):
        '''Returns list of method's arguments'''
        if hasattr(module, cls):
            cls_ref = getattr(module, cls)
            if cls_ref and hasattr(cls_ref, method):
                method_ref = getattr(cls_ref, method)
                if method_ref:
                    try:
                        res = getattr(inspect.getargspec(method_ref), 'args')
                        for item in exclude:
                            if(res.__contains__(item)):
                                res.remove(item)
                        if not res: res = []
                        if drop_self and res.__contains__('self'):
                            res.remove('self')
                        if get_varargs:
                            varargs_res = getattr(inspect.getargspec(method_ref), 'varargs')
                            if varargs_res: res.append(varargs_res)
                        if get_keywords:
                            keywords_res = getattr(inspect.getargspec(method_ref), 'keywords')
                            if keywords_res: res.append(keywords_res)
                        return res
                    except:
                        pass
        return []

    @staticmethod
    def get_method_params(module, clazz, method, holder=OrderedDict(), expendNestedTypes=False, groupOptions=False):
        args = MethodHelper.getMethodArgs(module, clazz, method)
        expender = MethodHelper.__expend_nested_type

        if not groupOptions:
            if args:
                if len(args) == 3:
                    if expendNestedTypes:
                        cand = expender(args[1], module, method, [])
                    else:
                        cand = args[1]

                    cand = ", ".join(cand) if type(cand) == list else cand

                    if not holder.has_key(args[2]):
                        holder[args[2]] = cand
                    else:
                        if holder[args[2]] == None:
                            if args[1] != None:
                                holder[args[2]] = 'None, ' + cand
                        else:
                            holder[args[2]] = holder[args[2]] + ', ' + cand
                elif len(args) == 2:
                    if not holder.has_key(args[1]):
                        holder[args[1]] = None
                    else:
                        if holder[args[1]] == None:
                            if holder[args[1]] != None:
                                holder[args[1]] = 'None' + ', ' + holder[args[1]]
                        else:
                            holder[args[1]] = holder[args[1]] + ', ' + 'None'
            return MethodHelper.__remove_replications(holder)
        else:
            if args:
                if len(args) == 3:
                    if expendNestedTypes:
                        cand = expender(args[1], module, method, [])
                    else:
                        cand = args[1]
                    cand = cand if type(cand) == list else [cand]

                    if not holder.has_key(args[2]):
                        holder[args[2]] = [cand]
                    else:
                        if holder[args[2]] == None:
                            if args[1] != None:
                                holder[args[2]] = [[], cand]
                        elif cand not in holder[args[2]]:
                            holder[args[2]].append(cand)
                elif len(args) == 2:
                    if not holder.has_key(args[1]):
                        holder[args[1]] = [[]]
                    else:
                        if holder[args[1]] == None:
                            if holder[args[1]] != None:
                                holder[args[1]] = [[], holder[args[1]]]
                        elif [] not in holder[args[1]]:
                            holder[args[1]].append([])

    @staticmethod
    def __remove_replications(holder):
        for k, v in holder.items():
            if v:
                new_val = []
                for item in v.split(', '):
                    if item and item not in new_val:
                        new_val.append(item)
                holder[k] = ", ".join(sorted(new_val))

    @staticmethod
    def __expend_nested_type(type_name, module, method, expended_types):
        """Recursive method's args types resolution"""
        getMethodArgs = MethodHelper.getMethodArgs
        expend_nested_type = MethodHelper.__expend_nested_type

        from ovirtcli.utils.typehelper import TypeHelper
        typ = TypeHelper.getDecoratorType(type_name)

        if typ:
            expended_types_tmp = getMethodArgs(module, typ, method, drop_self=True)
            if len(expended_types_tmp) > 1:
                for item in expended_types_tmp:
                    expend_nested_type(item, module, method, expended_types)
            elif len(expended_types_tmp) > 0:
                expended_types.append(expended_types_tmp[0])
        else:
            expended_types.append(type_name)
        return expended_types

    @staticmethod
    def get_documented_arguments(method_ref, as_params_collection=False, spilt_or=False):
        """Return a list of documented arguments for specific method, ignoring metadata."""
        return MethodHelper.get_arguments_documentation(method_ref, as_params_collection, spilt_or, ignore_non_args=True)


    @staticmethod
    def get_arguments_documentation(method_ref, as_params_collection=False, spilt_or=False, ignore_non_args=False):
        """Return a list of documented arguments with arguments metadata for specific method."""

        PARAM_ANNOTATION = '@param'
        IVAR_ANNOTATION = '@ivar'
        params_list = []
        ivars_list = []
        PERIOD_SYMBOL = '-'

        if method_ref and method_ref.__doc__:
            doc = method_ref.__doc__
            params_arr = doc.split('\n')

            for var in params_arr:
                if not ignore_non_args and var.find('Overload') != -1:
                    params_list.append('\n' + var.strip() + MethodHelper.NON_ARG_TEMPLATE + '\n')
                elif '' != var and var.find(PARAM_ANNOTATION) != -1:
                    splitted_line = var.strip().split(' ')
                    if len(splitted_line) >= 2:
                        prefix = splitted_line[0].replace((PARAM_ANNOTATION + ' '),
                                                          '--').replace(PARAM_ANNOTATION, '--')
                        param = splitted_line[1].replace('**', '')

                        if len(splitted_line) > 3 and splitted_line[3].startswith('('):
                            typ = ' '.join(splitted_line[2:3])
                            if prefix.startswith('['):
                                typ = typ + ']'
                        else:
                            typ = ' '.join(splitted_line[2:])

                        if param.find('.') != -1:
                            splitted_param = param.split('.')
                            new_param = PERIOD_SYMBOL.join(splitted_param[1:])
                            param = new_param

#                        params_hash[param.replace(':', '')] = splitted_line[1].replace(':', '') \
#                                                                              .replace('id|name', 'name')
                        if as_params_collection:
                            if spilt_or and param.find('|') != -1:
                                spl = param.replace(':', '').split('|')
                                if len(spl) == 2:
                                    p1 = spl[0]
                                    if spl[0].find(PERIOD_SYMBOL) != -1 :
                                        spl_ = spl[0].split('-')
                                        p2 = '-'.join(spl_[:len(spl_) - 1]) + '-' + spl[1]
                                    else:
                                        p2 = spl[1]

                                    params_list.append(p1)
                                    params_list.append(p2)
                                else:
                                    params_list.append(param.replace(':', ''))
                            else:
                                params_list.append(param.replace(':', ''))
                        else:
                            if spilt_or and param.find('|') != -1:
                                spl = param.split('|')
                                if len(spl) == 2:
                                    p1 = spl[0]
                                    if spl[0].find(PERIOD_SYMBOL) != -1 :
                                        spl_ = spl[0].split('-')
                                        p2 = '-'.join(spl_[:len(spl_) - 1]) + '-' + spl[1]
                                    else:
                                        p2 = spl[1]

                                    params_list.append(prefix + p1 + ' ' + typ)
                                    params_list.append(prefix + p2 + ' ' + typ)
                                else:
                                    params_list.append(prefix + param + ' ' + typ)
                            else:
                                params_list.append(prefix + param + ' ' + typ)
                if params_list and params_list[-1].find('collection') != -1:
                    params_list.append('  {' + MethodHelper.NON_ARG_TEMPLATE)
                elif not ignore_non_args and '' != var and var.find(IVAR_ANNOTATION) != -1:
                    params_list.append('    ' + var.lstrip().replace(IVAR_ANNOTATION + ' ', '') + MethodHelper.NON_ARG_TEMPLATE)
                elif not ignore_non_args and '' != var and var.find('}') != -1:
                    params_list.append('  }' + MethodHelper.NON_ARG_TEMPLATE)

        return params_list

    @staticmethod
    def get_object_methods(obj, exceptions=[]):
        """Return a list of type actions."""
        actions = []

        dct = (obj if type(obj) == type else type(obj)).__dict__
        if dct and len(dct) > 0:
            for method in dct:
                if method not in exceptions and not method.startswith('_'):
                    actions.append(method)
        return actions



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import types


class VersionHelper():
    @staticmethod
    def to_string(version, exceptions=['*final']):
        """Converts version to string format"""
        str_ver = ''
        for item in version:
            if item not in exceptions:
                if types.IntType == type(item) or item.isdigit():
                    str_ver += str(int(item))
                else:
                    str_ver += item
                str_ver += "."
        return str_ver[:len(str_ver) - 1]



#
# Copyright (c) 2013 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import pkg_resources

from ovirtcli.shell.capabilitiescmdshell import CapabilitiesCmdShell
from ovirtcli.command.command import OvirtCommand

class CapabilitiesCommand(OvirtCommand):

    name = 'capabilities'
    description = 'displaying system capabilities'
    args_check = (0, 1)
    valid_options = [ (
          '--' + item, str
         )
         for item in CapabilitiesCmdShell.OPTIONS
    ]

    helptext = """\
        == Usage ==

        capabilities [command options]

        == Description ==

        Displaying system capabilities.

        == Arguments ==

         * [features] - Lists version features.
        """

    def execute(self):
        """executes "capabilities"."""
        opts = self.options

        for capabilities in self.context.get_capabilities():
            capabilities_version = pkg_resources.parse_version(
               "%s.%s.%s.%s" % (
                        capabilities.major,
                        capabilities.minor,
                        capabilities.version if hasattr(
                               capabilities, 'version'
                        ) else '0',
                        capabilities.revision if hasattr(
                               capabilities, 'revision'
                        ) else '0'
                )
            )

            if  capabilities_version[0] == self.context.backend_version[0] and \
                capabilities_version[1] == self.context.backend_version[1]:
                if '--features' in opts:
                    self.context.formatter.format(
                              self.context,
                              capabilities.features.feature,
                              show_all=True
                    )
                else:
                    # backup and restore unrelevant data to this context
                    # saves deepcopy of the object
                    caps_features = capabilities.features
                    caps_id = capabilities.id

                    capabilities.features = None
                    capabilities.id = None

                    self.context.formatter.format(
                              self.context,
                              capabilities
                    )

                    capabilities.features = caps_features
                    capabilities.id = caps_id

    def show_help(self):
        """Show help for "capabilities"."""

        stdout = self.context.terminal.stdout
        stdout.write(self.helptext)



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.command.command import OvirtCommand
from ovirtcli.infrastructure.settings import OvirtCliSettings
from ovirtcli.shell.engineshell import EngineShell


class HelpCommand(OvirtCommand):

    name = 'help'
    description = 'show help'
    args_check = None
    valid_options = [('*', str)]

    helptext = """\
        == Usage ==

        help
        help <command> <command attributes> [command options]

        == Description ==

        Show help about a command. If no command is specified, a general help
        section is displayed.
        """

    helptext1 = """\
        == Introduction ==

        Welcome to $product-shell. This program is an interactive command-line
        interface into $product Virtualization. You can type any command in 
        the interface below.

        == Available Commands ==

          $commands

        Type 'help <command>' for any of these commands to show detailed help.
        The first command you probably want to issue is 'help connect' to
        learn how to connect to a oVirt manager.

        == Command Syntax ==

        The general format for each command is:

          help <command> <command attributes> [command options]

        If arguments contain spaces or other reserved characters, you need to
        quote them. You can use single (') and double (") quotes for this. The
        difference between single and double quotes is that within single
        quotes you can't use any single quotes, while in double quotes you can
        use a double quote provided that you escape it with backslash ('\\').

        Options are always in the long form: --option [value]. The value can
        be optional or mandatory, depending on the command, see command help
        for details.

        In addition to the basic command form, the following functionality is
        available:

          * You can use the '<', '<<', '>' and '>>' characters to perform
            shell-like redirections to files in the file system.
          * The output of any command can be piped to a shell command with
            the '|' character.
          * Shell commands can be executed by typing a '!' or 'shell' at the 
            beginning of a line.
          * Comments start with '#' and end at the end of a line.
            
        == Examples ==
        
        See 'SUPPORTED HELP FORMATS' section under each command help.

        == Configuration File ==

        oVirt-Shell can be configured with configuration file, configuration file
        ( ~/.ovirtshellrc) automatically created under your home directory (if not exist)
        at first ovirt-shell execution.
        """

    def execute(self):
        args = self.arguments
        opts = self.options
        stdout = self.context.terminal.stdout
        if len(args) == 0:
            subst = {}
            if self.context.connection is not None:
                commands = self.get_commands()
            else:
                off_line_content = \
                          [ f
                            for f in self.get_commands()
                            if f.split(' ')[0] in  EngineShell.OFF_LINE_CONTENT
                          ]
                commands = off_line_content
            subst['commands'] = self.format_list(commands, sort=True)
            subst['product'] = OvirtCliSettings.PRODUCT
            helptext = self.format_help(self.helptext1, subst)
            stdout.write(helptext)
        else:
            name = args[0]
            args = args[1:]
            opts = opts.items()
            opts += [('--help', None)]
            command = self.context._create_command(name, args, opts)
            command.run(self.context)



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from ovirtcli.shell.cmdshell import CmdShell
from cli.executionmode import ExecutionMode


class FileCmdShell(CmdShell):
    NAME = 'file'

    def __init__(self, context, parser):
        CmdShell.__init__(self, context, parser)

    def do_file(self, arg):
        """\
        == Usage ==

        file FILE

        == Description ==

        This command reads commands from the FILE instead of stdin.

        == Examples ==

        file /home/test/script
        """

        try:
            self.context.mode = ExecutionMode.SCRIPT
            with open(arg) as script:
                for line in script:
                    line = self.owner.precmd(line)
                    self.owner.print_line(line)
                    self.owner.onecmd(line)
        except Exception, e:
            self._error(str(e))
        finally:
            self.context.mode = ExecutionMode.SHELL



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import os.path
import pwd


def get_home_dir():
    """Return the user's home directory."""
    home = os.environ.get('HOME')
    if home is not None:
        return home
    try:
        pw = pwd.getpwuid(os.getuid())
    except Exception:
        return None
    return pw.pw_dir

def local_settings_file(name):
    """"Return the local settings file for `name'."""
    home = get_home_dir()
    if home is None:
        return
    name = name.replace('-', '').replace('_', '')
    fname = os.path.join(home, '.%src' % name)
    return fname

def which(cmd):
    """Find a command `cmd' in the path."""
    if cmd.startswith('/') and os.access(cmd, os.X_OK):
        return cmd
    path = os.environ.get('PATH')
    path = path.split(os.pathsep)
    for dir in path:
        fname = os.path.join(dir, cmd)
        if os.access(fname, os.X_OK):
            return fname

def get_pager():
    """Return the platform specific pager."""
    pager = os.environ.get('PAGER')
    if pager is None and which('less'):
        pager = 'less -FRX'
    if pager is None and which('more'):
        pager = 'more'
    return pager



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from cli.command import Command
from ovirtcli.utils.typehelper import TypeHelper

from ovirtsdk.xml import params
from ovirtsdk.utils.parsehelper import ParseHelper
import uuid
from ovirtcli.utils.methodhelper import MethodHelper
from ovirtsdk.infrastructure import brokers
from ovirtsdk.utils.ordereddict import OrderedDict
import itertools
from cli.messages import Messages
import types
import re
import keyword
import __builtin__


class OvirtCommand(Command):
    """Base class for oVirt commands."""

    def check_connection(self):
        """ensure we have a connection."""
        if self.context.connection is None:
            self.error('not connected', help='try \'help connect\'')
        return self.context.connection

    def resolve_base(self, options):
        """resolves a base object from a set of '--type-identifier value' options."""
        collection_candidate = self.check_connection()
        parnet_candidate_locator = 0
        base = None

        parnet_candidates = [key for key in options.keys() if key.endswith('-identifier')]
        parnet_candidates_permutations = list(itertools.permutations(parnet_candidates))

        if parnet_candidates_permutations[0]:
            for combination in parnet_candidates_permutations:
                for item in combination:
                    key = item
                    val = options[key]
                    parnet_candidate_locator += 1
                    typename = key[2:-11]

                    coll = TypeHelper.to_plural(typename)
                    if not (TypeHelper.isKnownType(typename) or  TypeHelper.isKnownType(coll)):
                        self.error(Messages.Error.NO_SUCH_TYPE % typename)

                    if hasattr(collection_candidate, coll):
                        coll_ins = getattr(collection_candidate, coll)
                        if hasattr(coll_ins, 'get'):
                            if not val:
                                self.error(Messages.Error.INVALID_OPTION % key)
                            uuid_cand = self._toUUID(val)
                            if uuid_cand != None:
                                base = coll_ins.get(id=val)
                            else:
                                base = coll_ins.get(name=val)

                            if not base:
                                if len(combination) > parnet_candidate_locator:
                                    continue
                                else:
                                    self.error(Messages.Error.NO_SUCH_OBJECT % (typename, val))

                    if len(parnet_candidates) == parnet_candidate_locator:
                        return base
                    else:
                        collection_candidate = base

            self.error(Messages.Error.CANNOT_CONSTRUCT_COLLECTION_MEMBER_VIEW % str(parnet_candidates_permutations))

    def __try_parse(self, param):
        """INTERNAL: try parsing data to int"""
        try:
            if param and param[0] in ('-', '+') and param[1:].isdigit():
                return int(param)
            elif param and param.isdigit():
                return int(param)
            else: return param
        except:
            return param

    def __do_set_primitive_list_data(self, obj, prop, val):
        for param in self.__split_with_escape(str(val), delimiter=','):
            getattr(obj, prop).append(param)

    def __do_set_data(self, obj, prop, fq_prop, val):
        """INTERNAL: set data in to object based on 'prop' map segmentation"""
        props = prop.split('-')
        props_len = len(props)
        if props_len > 1 or (props_len == 1 and hasattr(obj, prop)):
            for i in range(props_len):
                if props[i] == 'type': props[i] = 'type_'
                if i == (props_len - 1) and hasattr(obj, props[i]) and type(getattr(obj, props[i])) != list:
                    self.__set_property(obj, props[i], val, fq_prop)
                    return
                if hasattr(obj, props[i]) and type(getattr(obj, props[i])) == list:
                        params_set_cand = ParseHelper.getXmlType(props[i])
                        if params_set_cand:
                            obj_params_set_cand = params_set_cand.factory()
                            root_obj_params_set_cand = obj_params_set_cand
                        else:
                            self.__do_set_primitive_list_data(obj, props[i], val)
                            return

                        if not val:
                            self.error(Messages.Error.INVALID_COLLECTION_BASED_OPTION_SYNTAX % prop)

                        for param in self.__split_with_escape(str(val), delimiter=','):
                            obj_params_set_cand = root_obj_params_set_cand
                            if not param.startswith(props[i] + '.'):
                                self.error(
                                       Messages.Error.INVALID_OPTION_SEGMENT % \
                                       (param, prop)
                                )
                            param_data = param.replace(props[i] + '.', '').split('=')
                            if len(param_data) == 2:
                                spplited_param_data = param_data[0].split('.')
                                for param_period in spplited_param_data:
                                    if spplited_param_data[-1] == param_period:
                                        if not hasattr(obj_params_set_cand, param_period):
                                            param_period = self.fixParamNameIfParamIsKeyword(param_period)
                                        if hasattr(obj_params_set_cand, param_period):
                                            if getattr(obj_params_set_cand, param_period) != None:
                                                getattr(obj, props[i]).append(obj_params_set_cand)
                                                obj_params_set_cand = params_set_cand.factory()
                                                root_obj_params_set_cand = obj_params_set_cand
                                            setattr(obj_params_set_cand, param_period,
                                                    self.__try_parse(param_data[1].strip()))
                                        else:
                                            self.error(Messages.Error.INVALID_OPTION_SEGMENT % \
                                                       (param_period, param))
                                    elif hasattr(obj_params_set_cand, param_period) and \
                                         getattr(obj_params_set_cand, param_period) == None:
                                        param_period_cand = ParseHelper.getXmlType(param_period)
                                        if param_period_cand:
                                            param_period_cand_obj = param_period_cand.factory()
                                            setattr(obj_params_set_cand, param_period, param_period_cand_obj)
                                            obj_params_set_cand = param_period_cand_obj
                                    elif hasattr(obj_params_set_cand, param_period):
                                        param_period_cand = ParseHelper.getXmlType(param_period)
                                        if param_period_cand:
                                            param_period_cand_obj = getattr(obj_params_set_cand, param_period)
                                            obj_params_set_cand = param_period_cand_obj
                                    else:
                                        self.error(Messages.Error.INVALID_OPTION_SEGMENT % \
                                                   (param_period, param))
                            else:
                                self.error(Messages.Error.INVALID_COLLECTION_BASED_OPTION_SYNTAX % prop)
                        getattr(obj, props[i]).append(obj_params_set_cand)
                elif hasattr(obj, props[i]):
                    content = getattr(obj, props[i])
                    if content is None:
                        cand1 = ParseHelper.getXmlType(props[i])
                        if cand1:
                            cand = cand1.factory()
                            setattr(obj, props[i], cand)
                            obj = cand
                    else:
                        obj = content
                else:
                    cand2 = ParseHelper.getXmlTypeInstance(props[i])
                    if cand2 and hasattr(obj, cand2):
                        cand1 = ParseHelper.getXmlType(cand2)
                        if cand1:
                            props[i] = cand2
                            cand = cand1.factory()
                            setattr(obj, props[i], cand)
                            obj = cand
                        else:
                            self.error(Messages.Error.NO_SUCH_TYPE % cand2)
                    else:
                        self.error(Messages.Error.INVALID_ARGUMENT_SEGMENT % props[i])
        else:
            self.__set_property(obj, prop, val, fq_prop)

    def fixParamNameIfParamIsKeyword(self, param):
        if param in dir(__builtin__) or keyword.iskeyword(param):
            return param + '_'
        return param

    def __set_property(self, obj, prop, val, fq_prop):
        """INTERNAL: set data in to property"""
        if hasattr(obj, prop):
            setattr(obj, prop, val)
        elif hasattr(obj, prop + '_'):
            setattr(obj, prop + '_', val)
        else:
            pass
#            self.error('%s is not valid argument.' % fq_prop)

    def update_object_data(self, obj, options, scope=None):
        """Updates object properties with values from `options'."""

        for key in options.keys():
            prop = key.replace('--', '')
            val = options[key]
            if not prop.endswith('-id') and prop.endswith('-identifier'): continue
            if type(val) == types.ListType:
                for item in val:
                    self.__do_set_data(obj=obj, prop=prop, fq_prop=key, val=item)
            else:
                self.__do_set_data(obj=obj, prop=prop, fq_prop=key, val=val)

        return obj

    def read_object(self):
        """If input was provided via stdin, then parse it and return a binding
        instance."""
        stdin = self.context.terminal.stdin
        # REVISE: this is somewhat of a hack (this detects a '<<' redirect by
        # checking if stdin is a StringIO)
        if not hasattr(stdin, 'len'):
            return
        buf = stdin.read()
        try:
            obj = params.parseString(buf)
        except Exception:
            self.error('could not parse input')
        return obj


    def _get_query_params(self, opts, query_arg='--query', kwargs_arg='--kwargs'):
        """INTERNAL: retrieves query and kwargs from attribute options"""
        query = opts[query_arg] if opts.has_key(query_arg) else None
        kw = {}
        empty = True
        if opts.has_key(kwargs_arg):
            if opts[kwargs_arg]:
                for item in opts[kwargs_arg].split(';'):
                    if item.find("=") != -1:
                        k, v = item.split('=')
                        kw[k.replace('-', '.')] = v
                        empty = False
                    elif empty:
                        self.error(Messages.Error.INVALID_KWARGS_FORMAT % item)
            else:
                self.error(Messages.Error.INVALID_KWARGS_CONTENT)
        mopts = {}
        for k, v in opts.iteritems():
            if k != query_arg and k != kwargs_arg and not k.endswith('-identifier'):
                mopts[k if not k.startswith('--') else k[2:]] = v
        kw.update(mopts)

        return query, kw


    def get_collection(self, typ, opts={}, base=None, context_variants=[]):
        """retrieves collection members"""
        self.check_connection()
        connection = self.context.connection
        query, kwargs = self._get_query_params(opts)

        if base is None:
            if hasattr(connection, typ):
                options = self.get_options(method='list', resource=TypeHelper.to_singular(typ), as_params_collection=True)

                if query and 'query' not in options:
                    self.error(Messages.Error.NO_QUERY_ARGS)
                if kwargs and 'kwargs' not in options:
                    self.error(Messages.Error.NO_KWARGS % 'list')

                if query and kwargs:
                    return getattr(connection, typ).list(query=query, **kwargs)
                if query:
                    return getattr(connection, typ).list(query=query)
                if kwargs:
                    return getattr(connection, typ).list(**kwargs)
                return getattr(connection, typ).list()
        else:
            if hasattr(base, typ):
                options = self.get_options(method='list', resource=getattr(base, typ), as_params_collection=True)

                if query and 'query' not in options:
                    self.error(Messages.Error.NO_QUERY_ARGS)
                if kwargs and 'kwargs' not in options:
                    self.error(Messages.Error.NO_KWARGS % 'list')

                if query and kwargs:
                    return getattr(base, typ).list(query=query, **kwargs)
                if query:
                    return getattr(base, typ).list(query=query)
                if kwargs:
                    return getattr(base, typ).list(**kwargs)
                return getattr(base, typ).list()

        err_str = Messages.Error.NO_SUCH_COLLECTION
        if context_variants:
            err_str = err_str + (Messages.Info.POSSIBALE_ARGUMENTS_COMBINATIONS %
                                 str(context_variants))
        self.error(err_str % typ)

    def validate_options(self, opts, options):
        for opt in opts.keys():
            if opt.startswith('--'):
                opt_item = opt[2:]
            else: opt_item = opt
            if opt_item not in options and not opt_item.endswith('-identifier'):
                self.error(Messages.Error.NO_SUCH_OPTION % opt)

    def get_object(self, typ, obj_id, base=None, opts={}, context_variants=[]):
        """Return an object by id or name."""
        self.check_connection()
        connection = self.context.connection

        if base:
            options = self.get_options(method='get', resource=base,
                                       as_params_collection=True,
                                       context_variants=context_variants)
        else:
            options = self.get_options(method='get', resource=typ,
                                       as_params_collection=True,
                                       context_variants=context_variants)
            base = connection

        self.validate_options(opts, options)

        candidate = typ if typ is not None and isinstance(typ, type('')) \
                        else type(typ).__name__.lower()

        if hasattr(base, TypeHelper.to_plural(candidate)):
            coll = getattr(base, TypeHelper.to_plural(candidate))
        else:
            err_str = Messages.Error.NO_SUCH_TYPE_OR_ARS_NOT_VALID
            if context_variants:
                err_str = err_str + (Messages.Info.POSSIBALE_ARGUMENTS_COMBINATIONS
                                      % str(context_variants))
            self.error(err_str % candidate)

        if obj_id is not None:
            obj_uuid = self.__produce_identifier(obj_id)
            _, kwargs = self._get_query_params(opts)
            if obj_uuid is not None and 'id' in options:
                return self.__get_by_id(coll, obj_id, kwargs)
            if 'name' in options:
                return self.__get_by_name(coll, obj_id, kwargs)
            if 'alias' in options:
                return self.__get_by_alias(coll, obj_id, kwargs)
            return None

        if 'id' in options:
            obj_id, kwargs = self._get_query_params(opts, query_arg='--id')
            if obj_id is not None:
                return self.__get_by_id(coll, obj_id, kwargs)

        if 'name' in options:
            obj_id, kwargs = self._get_query_params(opts, query_arg='--name')
            if obj_id is not None:
                return self.__get_by_name(coll, obj_id, kwargs)

        if 'alias' in options:
            obj_id, kwargs = self._get_query_params(opts, query_arg='--alias')
            if obj_id is not None:
                return self.__get_by_alias(coll, obj_id, kwargs)

        self.error(Messages.Error.NO_ID % 'show')

    def __get_by_alias(self, coll, alias, kwargs):
        if 'alias' in kwargs:
            del kwargs['alias']
        if kwargs:
            return coll.get(alias=alias, **kwargs)
        else:
            return coll.get(alias=alias)

    def __get_by_name(self, coll, name, kwargs):
        if 'name' in kwargs:
            del kwargs['name']
        if kwargs:
            return coll.get(name=name, **kwargs)
        else:
            return coll.get(name=name)

    def __get_by_id(self, coll, id, kwargs):
        if 'id' in kwargs:
            del kwargs['id']
        if kwargs:
            return coll.get(id=id, **kwargs)
        else:
            return coll.get(id=id)

    def __produce_identifier(self, candidate):
        if type(candidate) == str:
            match = re.search(r'[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}', candidate)
            if match:
                return self._toUUID(candidate)
            match = re.search(r'[-+]?\d+', candidate)
            if match:
                return self._toLong(candidate)
        elif type(candidate) == int or type(candidate) == long:
            return candidate
        return None

    def _toLong(self, string):
        try:
            return long(string)
        except:
            return None

    def _toUUID(self, string):
        try:
            return uuid.UUID(string)
        except:
            return None

    def get_singular_types(self, method, typ=None, expendNestedTypes=True, groupOptions=True):
        """Return a list of singular types."""
        typs = TypeHelper.get_types_by_method(False, method, expendNestedTypes, groupOptions)
        if typ:
            if typs.has_key(typ):
                return typs[typ]
            else: return []
        else:
            return typs

    def get_plural_types(self, method, typ=None, expendNestedTypes=True, groupOptions=True):
        """Return a list of plural types."""
        typs = TypeHelper.get_types_by_method(True, method, expendNestedTypes, groupOptions)
        if typ:
            if typs.has_key(typ):
                return typs[typ]
            else: return []
        else:
            return typs

    def get_options(self, method, resource, sub_resource=None, as_params_collection=False, context_variants=[]):
        """Return a list of options for typ/action."""

        method_ref = None
        connection = self.check_connection()

        if isinstance(resource, type('')):
            if not sub_resource:
                    if resource and hasattr(connection, TypeHelper.to_plural(resource)) and \
                       type(getattr(connection, TypeHelper.to_plural(resource))).__dict__.has_key(method):
                        method_ref = getattr(getattr(connection,
                                                     TypeHelper.to_plural(resource)),
                                             method)
            else:
                if hasattr(sub_resource, TypeHelper.to_plural(resource)) and \
                type(getattr(sub_resource, TypeHelper.to_plural(resource))).__dict__.has_key(method):
                    method_ref = getattr(getattr(sub_resource,
                                                 TypeHelper.to_plural(resource)),
                                         method)
                elif hasattr(sub_resource, TypeHelper.to_plural(resource)) and \
                hasattr(brokers, TypeHelper.to_singular(type(getattr(sub_resource,
                                                             TypeHelper.to_plural(resource))).__name__)) and \
                hasattr(getattr(brokers, TypeHelper.to_singular(type(getattr(sub_resource,
                                                                     TypeHelper.to_plural(resource))).__name__)),
                        method):
                    method_ref = getattr(getattr(brokers,
                                                 TypeHelper.to_singular(type(getattr(sub_resource,
                                                                             TypeHelper.to_plural(resource))).__name__)),
                                         method)

            if not method_ref:
                err_str = Messages.Error.NO_SUCH_CONTEXT
                if context_variants:
                    err_str = err_str + (Messages.Info.POSSIBALE_ARGUMENTS_COMBINATIONS
                                         % str(context_variants))
                self.error(err_str % resource)

        elif isinstance(resource, brokers.Base):
            if hasattr(resource, method):
                method_ref = getattr(resource, method)
            elif hasattr(brokers, TypeHelper.to_plural(type(resource).__name__)) and \
            hasattr(getattr(brokers, TypeHelper.to_plural(type(resource).__name__)), method):
                method_ref = getattr(getattr(brokers, TypeHelper.to_plural(type(resource).__name__)), method)

        return MethodHelper.get_arguments_documentation(method_ref, as_params_collection)

    def is_supported_type(self, types, typ):
        if typ not in types:
            self.error(Messages.Error.NO_SUCH_TYPE % typ)
            return False
        return True

    def execute_method(self, resource, method_name, opts={}):
        """executes given method with specified opts."""

        if hasattr(resource, method_name):
            method = getattr(resource, method_name)

            method_args = OrderedDict().fromkeys(MethodHelper.getMethodArgs(brokers,
                                                                            method.im_class.__name__,
                                                                            method_name,
                                                                            drop_self=True))

            if method_args:
                for arg in method_args.keys():
                    param_type = ParseHelper.getXmlType(arg)
                    if param_type:
                        method_args[arg] = self.update_object_data(param_type.factory(), opts)
                    elif opts.has_key('--' + arg):
                        method_args[arg] = opts['--' + arg]
                    else:
                        # TODO: throw error if param is mandatory
                        pass

                try:
                    result = method(**method_args)
                except AttributeError:
                    self.error(Messages.Error.UNSUPPORTED_ATTRIBUTE)
            else:
                result = method()
            return result
        else:
            self.error('no such action "%s"' % method_name)

    def _get_action_methods(self, obj):
        """INTERNAL: return a list of type actions."""

        return MethodHelper.get_object_methods(obj, exceptions=['delete', 'update'])

    def __split_with_escape(self, text, delimiter=',', escape='\\'):
        """
        Splits a string so that the delimiter is ignored if it is preceded by
        the escape character.
        """

        chunks = []
        chunk = ''
        escaping = False
        for c in list(text) + [None]:
            if not escaping:
                if c is None:
                    chunks.append(chunk)
                    chunk = ''
                    escaping = False
                elif c == escape:
                    escaping = True
                elif c == delimiter:
                    chunks.append(chunk)
                    chunk = ''
                    escaping = False
                else:
                    chunk += c
                    escaping = False
            else:
                if c is None:
                    chunk += escape
                    chunks.append(chunk)
                    chunk = ''
                    escaping = False
                elif c == escape:
                    chunk += escape
                    escaping = False
                elif c == delimiter:
                    chunk += c
                    escaping = False
                else:
                    chunk += escape
                    chunk += c
                    escaping = False
        return chunks



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import os
import urllib

from cli.error import Error
from ovirtcli.platform import util
from cli.messages import Messages


def launch_spice_client(host, host_subject, port, secport, ticket, certurl, title,
                        debug=False):
    """Launch the SPICE client."""
    display = os.environ.get('DISPLAY')
    if display is None:
        raise Error, Messages.Error.CANNOT_START_CONSOLE_CLIENT % 'SPICE'
    cmd = util.which('spicec')
    if cmd is None:
        cmd = util.which('/usr/libexec/spicec')
    if cmd is None:
        raise Error, Messages.Error.NO_CONSOLE_FOUND % ('spice', 'spice')
    certdir = os.path.join(util.get_home_dir(), '.spicec')
    try:
        os.stat(certdir)
    except OSError:
        os.mkdir(certdir)
    certfile = os.path.join(certdir, 'spice_truststore.pem')
    try:
        os.stat(certfile)
    except OSError:
        certtmp = '%s.%d-tmp' % (certfile, os.getpid())
        urllib.urlretrieve(certurl, certtmp)
        os.rename(certtmp, certfile)
    args = ['spicec']
    if cmd.startswith('/usr/libexec'):
        args.extend([host])
        args.extend([str(port)])
        if secport:
            args.extend([str(secport)])
            args.extend(['--ssl-channels', 'smain,sinputs'])
            args.extend(['--ca-file', certfile])
            if host_subject and host_subject != '':
                args.extend(['--host-subject', host_subject])
        args.extend(['-p', ticket])
    else:
        args.extend(['-h', host])
        args.extend(['-p', str(port)])
        if secport:
            args.extend([ '-s', str(secport) ])
            args.extend(['--ca-file', certfile])
            if host_subject and host_subject != '':
                args.extend(['--host-subject', host_subject])
        args.extend(['-w', ticket])
        args.extend(['-t', title])
    pid, pstdin = util.spawn(cmd, args, debug)



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


class ColorHelper():
    BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)

    __PROMPT_PREFIX = "\001"
    __PREFIX = "\033[1;%dm"
    __SUFFIX = "\033[1;m"
    ___PROMPT_SUFFIX = "\002"
    __OFFSET = 30

    @staticmethod
    def colorize(text, color, is_prompt=False):
        """
        Colors text

        @param text: text to color
        @param color: color to use (ColorHelper.RED|ColorHelper.BLUE...)
        @param param: is_prompt indicates that should be performed prompt
                      unique treatment
        """
        if color:
            if is_prompt:
                return (
                       ColorHelper.__PROMPT_PREFIX + \
                       ColorHelper.__PREFIX + \
                       ColorHelper.___PROMPT_SUFFIX
                       ) % \
                       (ColorHelper.__OFFSET + color) + \
                       text + \
                       (
                       ColorHelper.__PROMPT_PREFIX + \
                       ColorHelper.__SUFFIX + \
                       ColorHelper.___PROMPT_SUFFIX
                       )
            return ColorHelper.__PREFIX % \
                   (ColorHelper.__OFFSET + color) + \
                   text + \
                   ColorHelper.__SUFFIX
        return text



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import os

from cli.command.command import Command


class CdCommand(Command):

    name = 'cd'
    alias = ('chdir',)
    description = 'change directory'
    args_check = 1
    helptext = """\
        == Usage ==

        cd <directory>

        == Description ==

        Change the current directory to <directory>.
        """

    def execute(self):
        dirname = self.arguments[0]
        try:
            os.chdir(dirname)
        except OSError, e:
            self.error('%s: %s' % (dirname, e.strerror))



#
# Copyright (c) 2013 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.state.dfsaevent import DFSAEvent
from ovirtcli.state.dfsastate import DFSAState
from ovirtcli.state.finitestatemachine import FiniteStateMachine

"""
    =========== define self instance ===========

    this is singleton instance of FiniteStateMachine
    that will be used across the app to maintain the
    state.

    =========== state optimization =============

    q ~  q` = {final states}, {other states}
        o

    q ~    q` <=> q~  q` and l(q,a) ~  l(q`,a)
       k+1          k                k

    ~   = ~ => ~  = ~
     k+1   k    k

    e.g:

        ---     =====     =====
      ->|1|-a-->||2||-a-->||3||----
        ---     =====     =====   |
         |      |   ^     |   ^   b
         b      |_b_|     |_a_|   |
         |        ^               |
         |        |               |
         v        a               |
        ---     __|__           =====
      ->|4|<--b-| 5 |<--------b-||6||-a--
      | ---     -----           =====   |
      |  |                        ^     |
      | a,b                       |_____|
      |__|


       1 2 3 4 5 6
      ------------
    a |2 3 3 4 2 6     ~  = {1,4,5}, {2,3,6}
    b |4 2 6 4 4 5      o
                       ~  = {1,5}, {2,3}. {4}, {6}
                        1

        ----      ====      ====
      ->|1,5|-a---||2||-a---||3||-b---
        ----      ====      ====     |
        |  ^      ^  |      ^   |    |
        b  |      |  b      |   a    |
        |  |      |__|      |___|    |
        |  |                         |
        |  |      ====               |
        |  -----b-||6||---------------
        V         ====
       ---        ^   |
       |4|-a,b-   |   a
       ---     |  |___|
        ^      |
        |      |
        |______|

"""

StateMachine = FiniteStateMachine(
    events=[
        DFSAEvent(
          name='exiting',
          sources=[
               DFSAState.CONNECTING,
               DFSAState.CONNECTED,
               DFSAState.DISCONNECTED,
               DFSAState.UNAUTHORIZED,
               DFSAState.COMMUNICATION_ERROR
          ],
          destination=DFSAState.EXITING,
          callbacks=[]),
        DFSAEvent(
          name='disconnecting',
          sources=[
               DFSAState.CONNECTED,
               DFSAState.UNAUTHORIZED,
               DFSAState.EXITING
          ],
          destination=DFSAState.DISCONNECTING,
          callbacks=[]),
        DFSAEvent(
          name='disconnected',
          sources=[
               DFSAState.DISCONNECTING,
               DFSAState.CONNECTING
          ],
          destination=DFSAState.DISCONNECTED,
          callbacks=[]),
        DFSAEvent(
          name='connecting',
          sources=[
               DFSAState.DISCONNECTED,
               DFSAState.UNAUTHORIZED,
               DFSAState.COMMUNICATION_ERROR
          ],
          destination=DFSAState.CONNECTING,
          callbacks=[]),
        DFSAEvent(
          name='connected',
          sources=[
               DFSAState.CONNECTING
          ],
          destination=DFSAState.CONNECTED,
          callbacks=[]),
        DFSAEvent(
          name='unauthorized',
          sources=[
               DFSAState.CONNECTED,
               DFSAState.CONNECTING,
               DFSAState.DISCONNECTING,
               DFSAState.COMMUNICATION_ERROR
          ],
          destination=DFSAState.UNAUTHORIZED,
          callbacks=[]),
        DFSAEvent(
          name='communication_error',
          sources=[
               DFSAState.CONNECTED,
               DFSAState.CONNECTING,
               DFSAState.DISCONNECTING
          ],
          destination=DFSAState.COMMUNICATION_ERROR,
          callbacks=[])
    ]
)



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.command.command import OvirtCommand
from ovirtcli.utils.typehelper import TypeHelper

class ListCommand(OvirtCommand):

    name = 'list'
    aliases = ('search',)
    description = 'list or search objects'
    usage = 'list <type> [search]... [options]'
    args_check = lambda self, x: len(x) > 0
    valid_options = [ ('*', str) ]
    SHOW_ALL_KEY = '--show-all'

    helptext = """\
        == Usage ==
    
        list <type> [parent identifiers] [command options]

        == Description ==

        List or search for objects of a certain type. There are two forms. If
        only <type> is provided, all objects of the specified type are
        returned. If a search query is given, it must be a valid oVirt search
        query. In that case objects matching the query are returned.

        == Supported Help formats ==

        - This help will list all available attribute options for listing 
          collection of given types
          
          * format      - help list types
          * example     - help list vms

        - This help will list all available attribute options for listing 
          subcollection of given types
          
          * format      - help list subtypes --parent-identifier
          * example     - help list disks --vm-identifier myvm

        == Available Types ==

        The <type> parameter must be one of the following.

          $types

        == Object Identifiers ==

        Some objects can only exist inside other objects. For example, a disk
        can only exist in the content of a virtual machine. In this case, one
        or more object identifier opties needs to be provided to identify the
        containing object.

        An object identifier is an option of the form '--<type>id <id>'. This
        would identify an object with type <type> and id <id>. See the
        examples section below for a few examples.

        == Examples ==

        - This example lists all virtual machines:

          $ list vms

        - This example lists all virtual machines with all (not empty) properties,
          (by default only id/name/description properties displayed, using
          --show-all option, all not empty properties will be displayed,
          to see entire resource - use 'show' command)

          $ list vms --show-all

        - This example lists only virtual machines that have a name that starts 
          with "myvm":

          $ list vms --query "name=myvm*"

        - This example lists all virtual machines that have a given description:

          $ list vms --query 'description="The description"'

          Note that the the syntax of the search queries requires double quotes
          when the text includes spaces.

        - This example list all disks by vm_id in virtual machine 'myvm':

          $ list disks --vm-identifier myvm
          
        - This example list all vms having memory size of 1073741824 using client 
          side filtering (this kind of filtering is useful on non queryable collections
          or for filtering resources based on properties which are not supported by oVirt
          querying dialect)
    
          $ list vms --kwargs "memory=1073741824"
          
        - This example retrieves vm disk with name 'Disk 3' using client side filtering 
          as oVirt dialect is not available on vm disks collection.          
          
          $ list disks --vm-identifier myvm --kwargs "name=Disk 3"

        == Return values ==

        This command will exit with one of the following statuses. To see the
        exit status of the last command, type 'status'.

          $statuses
        """

    helptext1 = """\
        == Usage ==

        - list <type>
            
        - list <type> <id> [parent identifiers] [command options]

        == Description ==

        Lists an objects with type '$type'. See 'help list' for generic
        help on listing objects.

        == Attribute Options ==

        The following options are available for objects with type '$type':

          $options

        == Return values ==

        This command will exit with one of the following statuses. To see the
        exit status of the last command, type 'status'.

          $statuses
        """

    def execute(self):
        """Execute "list"."""
        args = self.arguments
        opts = self.options

        typs = self.get_plural_types(method='list', typ=args[0])
        self.context.formatter.format(
                      self.context,
                      self.get_collection(
                          typ=args[0],
                          opts=opts,
                          base=self.resolve_base(opts),
                          context_variants=typs
                      ),
                      show_all=True if opts and opts.has_key(
                                                  ListCommand.SHOW_ALL_KEY
                                                )
                                    else False
        )

    def show_help(self):
        """Show help for "list"."""
        self.check_connection()
        args = self.arguments
        opts = self.options

        subst = {}
        types = self.get_plural_types(method='list')

        if not args or self.is_supported_type(types.keys(), args[0]):
            subst['types'] = self.format_map(types)

            statuses = self.get_statuses()
            subst['statuses'] = self.format_list(statuses)

            if len(args) == 1 and len(opts) == 1:
                helptext = self.helptext
                subst['types'] = self.format_map({
                                  args[0]:types[args[0]]
                                 }
                )
                subst['type'] = args[0]
            elif len(args) == 1 and len(opts) > 1:
                helptext = self.helptext1
                params_list = self.get_options(
                               method='list',
                               resource=TypeHelper.to_singular(args[0]),
                               sub_resource=self.resolve_base(opts),
                               context_variants=types[args[0]]
                )
                subst['options'] = self.format_list(params_list)
                subst['type'] = args[0]
            else:
                helptext = self.helptext

            helptext = self.format_help(helptext, subst)
            self.write(helptext)



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from cli.command.command import Command


class SetCommand(Command):

    name = 'set'
    description = 'set a configuration variable'
    args_check = (0, 2)
    helptext = """\
        == Usage ==

        set
        set <name> <value>

        == Description ==

        Set or show configuration variables. This command has two forms. If
        called without any arguments, it will show a list of all current
        configuration variables. If it is called with a <name> and <value>
        argument, the configuration variable <name> is set to <value>.
        """

    def execute(self):
        args = self.arguments
        settings = self.context.settings
        stdout = self.context.terminal.stdout
        if len(args) == 0:
            names = settings.keys()
            names.sort()
            stdout.write('Current settings:\n\n')
            for name in names:
                stdout.write('  %s = %s\n' % (name, repr(settings[name])))
            stdout.write('\n')
        elif len(args) == 2:
            key, value = args
            try:
                settings[key] = value
            except (KeyError, ValueError), e:
                self.error(e.message)



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

class AutoCompletionHelper(object):

    @staticmethod
    def __map_options(args, common_options=[], specific_options={}, specific_arguments={}):
        mp = {}
        for key in args.keys():
            if args[key] != None:
                vals = []
                for val in args[key].split(','):
                    if val and val != '': vals.append(
                          AutoCompletionHelper._resolve_value(
                                  val.strip(),
                                  common_options,
                                  specific_options,
                                  specific_arguments
                          )
                    )
                if common_options:
                    vals.extend(common_options[:])

                mp[key] = vals

                if specific_options.has_key(key):
                    mp[key].extend(specific_options[key])

                if specific_arguments.has_key(key):
                    mp[key].extend(specific_arguments[key])
            else:
                if common_options:
                    mp[key] = common_options[:]
                else:
                    mp[key] = []

                if specific_options.has_key(key):
                    mp[key].extend(specific_options[key])

                if specific_arguments.has_key(key):
                    mp[key].extend(specific_arguments[key])
        return mp

    @staticmethod
    def _resolve_value(val, common_options=[], specific_options={}, specific_arguments={}):
        if val == 'None': return val
        return val + (
              '-identifier' if val not in common_options \
                                and (not AutoCompletionHelper._is_verb_in_dict_values(
                                              specific_options, val
                                              )
                                    )
                                and (not AutoCompletionHelper._is_verb_in_dict_values(
                                              specific_arguments, val
                                              )
                                    )
                             else ' '
        )

    @staticmethod
    def _get_verb_replecations(container, text):
        times = 0
        for f in container:
            if text and f.startswith(text if not text.startswith('--')
                                                       else text.strip()[2:]):
                times += 1
                if times > 2: break
        return times

    @staticmethod
    def _is_verb_in_dict_values(dct, text):
        for key in dct.keys():
            if AutoCompletionHelper._get_verb_replecations(dct[key], text) > 0:
                return True
        return False

    @staticmethod
    def complete(line, text, args, common_options=[], specific_options={}, specific_arguments={}, all_options=False):
        '''
        Provides  auto-completion capabilities
        
        @param line: entered line
        @param text: appended text to complete
        @param args: dictionary of verbs and their's options
        @param common_options: common options to append for complete suggestions        
        @param specific_options: type specific options to append for complete suggestions
        @param specific_arguments: type specific arguments to append for complete suggestions
        '''

        mp = AutoCompletionHelper.__map_options(args, common_options, specific_options, specific_arguments)
        if not all_options:
            spl = line.split(' ')
            if len(spl) >= 2:
                s_text = text.strip()
                if len(spl) == 2 and text != ' ':
                    repl = AutoCompletionHelper._get_verb_replecations(mp.keys(), s_text)
                    i_completions = [ f + ' ' if text in mp.keys() or repl == 1 else f
                                    for f in mp.keys()
                                    if f.startswith(s_text if not s_text.startswith('--')
                                                       else text.strip()[2:])
                                    ]
                else:
                    obj = spl[1].strip()
                    repl = AutoCompletionHelper._get_verb_replecations(mp[obj], s_text)
                    i_completions = [('--' if (not AutoCompletionHelper._is_verb_in_dict_values(specific_arguments, f)) else '')
                                     + f + ' '
                                     if text in mp[obj] or repl == 1 or len(mp[obj]) == 1
                                                                     or (len(mp[obj]) == 2 and 'None' in mp[obj]) == 1
                                     else f
                                    for f in mp[obj]
                                    if f != 'None' and f.startswith(s_text  if not s_text.startswith('--')
                                                       else text.strip()[2:])
                                    ]

                return i_completions
            else:
                return mp.keys()[:]
        else:
            if not text:
                i_completions = args.keys()[:]
            else:
                s_text = text.strip()
                repl = AutoCompletionHelper._get_verb_replecations(args.keys(), text)
                i_completions = [ '--' + f + ' ' if repl == 1 or text in args.keys() else f
                                for f in args.keys()
                                if f.startswith(s_text if not s_text.startswith('--')
                                                       else text.strip()[2:])
                                ]

        return i_completions



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.command.command import OvirtCommand
from ovirtcli.infrastructure.settings import OvirtCliSettings
from cli.messages import Messages


class PingCommand(OvirtCommand):

    name = 'ping'
    description = 'test the connection'
    helptext = """\
        == Usage ==

        ping

        == Description ==

        Test the connection to the %s manager. This command will go out to
        the %s manager and retrieve a remote resource. If it succeeds, you
        know that the URL, username and password are OK (note: there are few 
        optional parameters that might be needed for establishing connection).
        """ % (OvirtCliSettings.PRODUCT , OvirtCliSettings.PRODUCT)

    def execute(self):
        connection = self.check_connection()
        try:
            connection.test(throw_exception=True)
        except Exception, e:
            self.write('\n' + str(e) + '\n')
            self.error(
                   Messages.Error.CANNOT_CONNECT_TO_BACKEND
                   %
                   OvirtCliSettings.PRODUCT
            )
        else:
            self.write(
                   Messages.Info.SUCESS_CONNECT_TO_BACKEND
            )



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ovirtcli.command.command import OvirtCommand
from ovirtcli.state.statemachine import StateMachine

from cli.context import ExecutionContext
from cli.messages import Messages

class DisconnectCommand(OvirtCommand):

    name = 'disconnect'
    description = 'disconnect from oVirt manager'
    helptext = """\
        == Usage ==

        disconnect

        == Description ==

        Disconnect an active connection to oVirt manager, if any. This method
        can be called multiple times. It is not an error to disconnect when
        not connected.
        """

    def execute(self):
        connection = self.context.connection
        if connection is None:
            self.error(
               Messages.Error.NOT_CONNECTED
            )
            return
        try:
            StateMachine.disconnecting()  # @UndefinedVariable
            connection.disconnect()
            self.context.status = ExecutionContext.OK
        except Exception:
            # TODO: consider rollback on disconnect failure (StateMachine.rollback())
            self.context.status = ExecutionContext.COMMAND_ERROR
        finally:
            self.context._clean_settings()
            self.context.connection = None
            StateMachine.disconnected()  # @UndefinedVariable



#
# Copyright (c) 2010 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#           http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from cli.plyparse import PLYParser
from cli.error import ParseError


class Parser(PLYParser):
    """A parser for CLI commands. The general form of a CLI comand is:

    <command> [parameter] [redirection] [pipeline]
    ! <command>

    A parameter can be an argument or an option.

    This parser supports quoted strings for arguments and options, and
    multi-line inputs.
    """

    tokens = ('WORD', 'STRING', 'OPTION', 'LT', 'LTLT', 'GT', 'GTGT',
              'BANG', 'PIPE', 'NEWLINE', 'MARKER', 'HEREDOC', 'SHELL')
    literals = ('=', ';')
    states = [('heredoc1', 'inclusive'), ('heredoc2', 'exclusive'),
              ('shell', 'exclusive')]

    t_LT = r'<'
    t_GT = r'>'
    t_GTGT = r'>>'

    t_ignore = ' \t'
    t_ignore_comment = r'\#.*'
    t_ignore_quoted_newline = r'\\\n'
    t_heredoc2_ignore = ' \t'
    t_shell_ignore = ' \t'

    # Note that the expression for options has to be before the
    # expression for words because the generated parser tries them in
    # order, and the expression for words is a supperset of the
    # expression for options. If the expression for words goes first
    # then options are never matched.

    def t_OPTION(self, t):
        r'-(-[a-zA-Z_][a-zA-Z0-9_]*)+'
        return t

    def t_WORD(self, t):
        r'[^ \n\t"\'<>|!\\#;]+'
        try:
            t.value = int(t.value)
        except ValueError:
            pass
        return t

    def t_STRING(self, t):
        r'''(?s)("([^"\\]|\\.)*"|'[^']*')'''
        if t.value[0] == '"':
            t.value = t.value[1:-1].replace(r'\\', '\\')
        else:
            t.value = t.value[1:-1]
        t.value = t.value.replace('\\\n', '')
        return t

    def t_LTLT(self, t):
        r'<<'
        t.lexer.begin('heredoc1')
        self.heredoc = []
        return t

    def t_BANG(self, t):
        r'!'
        t.lexer.begin('shell')
        return t

    def t_PIPE(self, t):
        r'\|'
        t.lexer.begin('shell')
        return t

    def t_NEWLINE(self, t):
        r'\n'
        return t

    def t_heredoc1_MARKER(self, t):
        r'[a-zA-Z0-9_-]+'
        self.marker = t.value
        return t

    def t_heredoc1_NEWLINE(self, t):
        r'\n'
        t.lexer.begin('heredoc2')

    def t_heredoc2_HEREDOC(self, t):
        r'.*\n'
        if t.value[:-1] != self.marker:
            self.heredoc.append(t.value)
            return None  # return the whole heredoc later as one token
        t.lexer.begin('INITIAL')
        t.value = ''.join(self.heredoc)
        t.lexer.lexpos -= 1  # read newline again
        return t

    def t_shell_SHELL(self, t):
        r'.*(?<!\\)\n'
        t.lexer.begin('INITIAL')
        t.value = t.value.replace('\\\n', '')[:-1]
        t.lexer.lexpos -= 1  # read newline again
        return t

    def t_error(self, t):
        if t.value[:1] in '"\'':
            raise EOFError
        raise ParseError

    t_heredoc2_error = t_error
    t_shell_error = t_error

    def p_main(self, p):
        """main : command
                | main command
        """
        if p[1] is None:
            p[0] = []
        elif len(p) == 2:
            p[0] = [p[1]]
        elif len(p) == 3:
            p[0] = p[1] + [p[2]]

    def p_command(self, p):
        """command : name parameter_list redirections pipeline heredoc eol
                   | BANG SHELL eol
                   | eol
        """
        if len(p) == 7:
            if p[5]:
                for i in range(len(p[3])):
                    if p[3][i][0] == '<<':
                        p[3][i] = ('<<', p[5])
                        break
            p[0] = (p[1], p[2], p[3], p[4])
        elif len(p) == 4:
            p[0] = (p[1], p[2])
        elif len(p) == 2:
            p[0] = None

    def p_name(self, p):
        """name : WORD"""
        p[0] = p[1]

    def p_parameter_list(self, p):
        """parameter_list : empty
                          | parameter_list parameter
        """
        if len(p) == 2:
            p[0] = []
        else:
            p[0] = p[1] + [p[2]]

    def p_parameter(self, p):
        """parameter : argument
                     | option
        """
        p[0] = p[1]

    def p_argument(self, p):
        """argument : WORD
                    | STRING
        """
        p[0] = p[1]

    def p_option(self, p):
        """option : OPTION option_value
                  | OPTION '=' option_value
        """
        if len(p) == 3:
            p[0] = (p[1], p[2])
        else:
            p[0] = (p[1], p[3])

    def p_option_value(self, p):
        """option_value : WORD
                        | STRING
                        | empty
        """
        p[0] = p[1]

    def p_redirections(self, p):
        """redirections : empty
                        | redirection
                        | redirections redirection
        """
        if len(p) == 2:
            if p[1] is None:
                p[0] = []
            else:
                p[0] = [p[1]]
        else:
            p[0] = p[1] + [p[2]]

    def p_redirection(self, p):
        """redirection : LT file
                       | LTLT MARKER
                       | GT file
                       | GTGT file
        """
        p[0] = (p[1], p[2])

    def p_pipeline(self, p):
        """pipeline : PIPE SHELL
                    | empty
        """
        if len(p) == 3:
            p[0] = p[2]

    def p_heredoc(self, p):
        """heredoc : HEREDOC
                   | empty
        """
        p[0] = p[1]

    def p_file(self, p):
        """file : WORD
                | STRING
        """
        p[0] = p[1]

    def p_eol(self, p):
        """eol : ';' NEWLINE
               | NEWLINE
               | ';'
        """
        p[0] = None

    def p_empty(self, p):
        """empty : """

    def p_error(self, p):
        if p is None:
            raise EOFError
        raise ParseError, 'syntax error'
